@@ -1,6 +1,6 @@
-## $Id: kildclient.pl,v 1.41 2008/03/23 21:02:07 ekalin Exp $
+## $Id: kildclient.pl 1027 2009-02-12 18:27:11Z ekalin $
 
-### Copyright (C) 2004-2008 Eduardo M Kalinowski <ekalin@gmail.com>
+### Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
 ###
 ### This program is free software; you can redistribute it and/or modify
 ### it under the terms of the GNU General Public License as published by
@@ -69,6 +69,67 @@ sub _ { return gettext($_[0]); }
   }
 
 
+  sub mlsend {
+    my $self = shift;
+
+    if (scalar @_ == 0) {
+      $self->echonl(sprintf(_('%s: Too few arguments.'), 'mlsend'));
+      return;
+    }
+
+    my $args = shift;
+    if ($args->{initialtext} && ref($args->{initialtext}) ne 'ARRAY') {
+      $args->{initialtext} = [ split(/\n/, $args->{initialtext}) ];
+    }
+
+    if ($args->{finaltext} && ref($args->{finaltext}) ne 'ARRAY') {
+      $args->{finaltext} = [ split(/\n/, $args->{finaltext}) ];
+    }
+
+    return $self->_mlsend($args);
+  }
+
+
+  sub sendlines {
+    my $self = shift;
+
+    if (scalar @_ == 0) {
+      $self->echonl(sprintf(_('%s: Too few arguments.'), 'sendlines'));
+      return;
+    }
+
+    my $args = { initialtext => $_[0] };
+    if (scalar @_ >= 2) {
+      $args->{delay} = $_[1];
+    }
+    if (scalar @_ >= 3) {
+      $args->{linesatime} = $_[2];
+    }
+
+    return $self->mlsend($args);
+  }
+
+
+  sub sendfile {
+    my $self = shift;
+
+    if (scalar @_ == 0) {
+      $self->echonl(sprintf(_('%s: Too few arguments.'), 'sendfile'));
+      return;
+    }
+
+    my $args = { file => $_[0] };
+    if (scalar @_ >= 2) {
+      $args->{delay} = $_[1];
+    }
+    if (scalar @_ >= 3) {
+      $args->{linesatime} = $_[2];
+    }
+
+    return $self->_mlsend($args);
+  }
+
+
   sub prev {
     my $self = shift;
     my $num = $_[0];
@@ -307,7 +368,7 @@ sub _ { return gettext($_[0]); }
           eval("\$deleted += \$self->_$function($target)");
         }
         if (!$deleted) {
-          $self->echonl(sprintf(_('No such %s'), $objname));
+          $self->echonl(sprintf(_('No such %s.'), $objname));
         }
       } else {
         my @found = reverse($self->_getobjectnumber($target,
@@ -315,7 +376,7 @@ sub _ { return gettext($_[0]); }
                                                     0,
                                                     $extrafirstarg));
         if (scalar @found == 0) {
-          $self->echonl(sprintf(_('No such %s'), $objname));
+          $self->echonl(sprintf(_('No such %s.'), $objname));
         } else {
           foreach (@found) {
             if ($extrafirstarg) {
@@ -328,45 +389,50 @@ sub _ { return gettext($_[0]); }
       }
 
       unless ($self->{SILENT}) {
-        if ($deleted == 1) {
-          $self->echonl(sprintf(_('1 %s deleted.'), $objname));
+        if (!$deleted) {
+          $self->echonl(sprintf(_('No %s deleted.'), $objnameplural));
+        } elsif ($deleted == 1) {
+          $self->echonl(sprintf(_('One %s deleted.'), $objname));
         } else {
           $self->echonl(sprintf(_('%d %s deleted.'), $deleted, $objnameplural));
         }
       }
     } else {
       $self->echonl(sprintf(_('%s: Too few arguments.'), $function));
+      $deleted = undef;
     }
+
+    return $deleted;
   }
 
 
   sub deltrigger {
     my $self = shift;
-    $self->_delobject($_[0], 'deltrigger', 0, _('trigger'), _('triggers'));
+    return $self->_delobject($_[0], 'deltrigger', 0, _('trigger'), _('triggers'));
   }
 
 
   sub delalias {
     my $self = shift;
-    $self->_delobject($_[0], 'delalias', 1, _('alias'), _('aliases'));
+    return $self->_delobject($_[0], 'delalias', 1, _('alias'), _('aliases'));
   }
 
 
   sub delmacro {
     my $self = shift;
-    $self->_delobject($_[0], 'delmacro', 2, _('macro'), _('macros'));
+    return $self->_delobject($_[0], 'delmacro', 2, _('macro'), _('macros'));
   }
 
 
   sub deltimer {
     my $self = shift;
-    $self->_delobject($_[0], 'deltimer', 3, _('timer'), _('timers'));
+    return $self->_delobject($_[0], 'deltimer', 3, _('timer'), _('timers'));
   }
 
 
   sub delhook {
     my $self  = shift;
-    $self->_delobject($_[1], 'delhook', 4, _('hook'), _('hooks'), $_[0]);
+    return $self->_delobject($_[1], 'delhook', 4, _('hook'), _('hooks'), $_[0]);
   }
 
 
