@@ -1,7 +1,7 @@
-/* $Id: cmdhistory.c,v 1.12 2008/03/23 21:09:46 ekalin Exp $ */
+/* $Id: cmdhistory.c 1029 2009-02-15 13:37:36Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2008 Eduardo M Kalinowski <ekalin@gmail.com>
+ * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -52,10 +52,6 @@ typedef struct CmdHistorySendData_s CmdHistorySendData;
  * Function prototypes *
  ***********************/
 static GtkWidget *create_cmd_history_dialog(GtkWindow *parent, World *world);
-static gint       inverse_order_sort_func(GtkTreeModel *model,
-                                          GtkTreeIter  *a,
-                                          GtkTreeIter  *b,
-                                          gpointer      userdata);
 static void       cmdhistory_recall_cb(GtkWidget *widget, gpointer data);
 static void       cmdhistory_start_send_cb(GtkWidget *widget, gpointer data);
 static gboolean   cmdhistory_send_timer_cb(gpointer data);
@@ -192,7 +188,6 @@ create_cmd_history_dialog(GtkWindow *parent, World *world)
   GladeXML         *gladexml;
   GtkWidget        *dlg;
   GtkTreeView      *viewCommands;
-  GtkTreeModel     *inverse_model;
   GtkTreeSelection *selection;
   GtkCellRenderer  *renderer;
   GtkWidget        *btnRecall;
@@ -212,16 +207,7 @@ create_cmd_history_dialog(GtkWindow *parent, World *world)
 
   /* Configure tree view */
   viewCommands = GTK_TREE_VIEW(glade_xml_get_widget(gladexml, "viewCommands"));
-  inverse_model = gtk_tree_model_sort_new_with_model(world->cmd_list);
-  gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(inverse_model),
-                                  0,
-                                  inverse_order_sort_func,
-                                  NULL,
-                                  NULL);
-  gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(inverse_model),
-                                       0, GTK_SORT_DESCENDING);
-  gtk_tree_view_set_model(viewCommands, inverse_model);
-  g_object_unref(inverse_model);
+  gtk_tree_view_set_model(viewCommands, world->cmd_list);
   renderer = gtk_cell_renderer_text_new();
   gtk_tree_view_insert_column_with_data_func(viewCommands,
                                              0,
@@ -235,11 +221,11 @@ create_cmd_history_dialog(GtkWindow *parent, World *world)
 
   /* Signals */
   glade_xml_signal_autoconnect(gladexml);
-  btnClose = glade_xml_get_widget(gladexml, "btnClose");
+  btnClose = glade_xml_get_widget(gladexml, "btnCHClose");
   g_signal_connect_swapped(G_OBJECT(btnClose), "clicked",
                            G_CALLBACK(gtk_widget_hide), dlg);
 
-  btnSend = glade_xml_get_widget(gladexml, "btnSend");
+  btnSend = glade_xml_get_widget(gladexml, "btnCHSend");
   g_signal_connect(G_OBJECT(btnSend), "clicked",
                    G_CALLBACK(cmdhistory_start_send_cb), world);
   btnSendClose = glade_xml_get_widget(gladexml, "btnSendClose");
@@ -264,7 +250,7 @@ create_cmd_history_dialog(GtkWindow *parent, World *world)
                    G_CALLBACK(cmdhistory_find_cb), world);
 
   /* Default values */
-  spnDelay = glade_xml_get_widget(gladexml, "spnDelay");
+  spnDelay = glade_xml_get_widget(gladexml, "spnCHDelay");
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(spnDelay),
                             globalPrefs.multi_cmd_delay);
   spnNCommands = glade_xml_get_widget(gladexml, "spnNCommands");
@@ -275,33 +261,6 @@ create_cmd_history_dialog(GtkWindow *parent, World *world)
 }
 
 
-static
-gint
-inverse_order_sort_func(GtkTreeModel *model,
-                        GtkTreeIter  *a,
-                        GtkTreeIter  *b,
-                        gpointer      userdata)
-{
-  /* Used to show the model in the inverse order (so that the oldest command,
-     last in the model, appears first.)
-     The return value of this function actually sorts them in normal
-     order, but since sorting is set as DESCENDING, the order is inverted
-     there.
-  */
-  GtkTreePath *path_a;
-  GtkTreePath *path_b;
-  gint         retval;
-
-  path_a = gtk_tree_model_get_path(model, a);
-  path_b = gtk_tree_model_get_path(model, b);
-  retval = gtk_tree_path_compare(path_a, path_b);
-  gtk_tree_path_free(path_a);
-  gtk_tree_path_free(path_b);
-  return retval;
-
-}
-
-
 static
 void
 cmdhistory_recall_cb(GtkWidget *widget, gpointer data)
@@ -313,7 +272,6 @@ cmdhistory_recall_cb(GtkWidget *widget, gpointer data)
   GtkTreeSelection *selection;
   GList            *selected_rows;
   gint              n_selected;
-  GtkTreeIter       inverse_iter;
   gchar            *command;
 
   gladexml = glade_get_widget_tree(widget);
@@ -353,11 +311,8 @@ cmdhistory_recall_cb(GtkWidget *widget, gpointer data)
 
   save_tentative_command(world);
   selected_rows = gtk_tree_selection_get_selected_rows(selection, &model);
-  gtk_tree_model_get_iter(model, &inverse_iter,
+  gtk_tree_model_get_iter(model, &world->current_cmd,
                           (GtkTreePath *) selected_rows->data);
-  gtk_tree_model_sort_convert_iter_to_child_iter(GTK_TREE_MODEL_SORT(model),
-                                                 &world->current_cmd,
-                                                 &inverse_iter);
   command = cmdhistory_get_command(world->cmd_list, &world->current_cmd);
   simo_combo_box_set_text(world->gui->cmbEntry, command);
   g_free(command);
@@ -397,7 +352,7 @@ cmdhistory_start_send_cb(GtkWidget *widget, gpointer data)
     return;
   }
 
-  spnDelay = glade_xml_get_widget(gladexml, "spnDelay");
+  spnDelay = glade_xml_get_widget(gladexml, "spnCHDelay");
   delay = gtk_spin_button_get_value(GTK_SPIN_BUTTON(spnDelay));
   spnNCommands = glade_xml_get_widget(gladexml, "spnNCommands");
 
@@ -648,12 +603,10 @@ add_recent_command(World *world, const gchar *command)
 {
   GtkTreeIter  previous;
   GtkTreeIter  most_recent;
+  gboolean     go_on;
+  gboolean     is_repeated = FALSE;
   gchar       *previous_command = NULL;
 
-  if (world->repeat_commands && world->cmd_just_selected_from_combo) {
-    insert_recent_command(world, command);
-  }
-
   if (!world->repeat_commands || world->cmd_has_been_edited
       || world->cmd_just_selected_from_combo) {
     gtk_tree_model_get_iter_first(world->cmd_list, &most_recent);
@@ -661,17 +614,31 @@ add_recent_command(World *world, const gchar *command)
                        CMDLIST_COL_TENTATIVE, NULL,
                        -1);
 
-    if (gtk_tree_model_iter_nth_child(world->cmd_list, &previous,
-                                      NULL, 1)) {
-      gtk_tree_model_get(world->cmd_list, &previous,
-                         CMDLIST_COL_COMMAND, &previous_command,
-                         -1);
+    /* Try to locate the same command in the history */
+    go_on = gtk_tree_model_iter_nth_child(world->cmd_list, &previous,
+                                          NULL, 1);
+    while (go_on) {
+      previous_command = cmdhistory_get_command(world->cmd_list, &previous);
+      if (previous_command && strcmp(command, previous_command) == 0) {
+        is_repeated = TRUE;
+        g_free(previous_command);
+        break;
+      }
+
+      go_on = gtk_tree_model_iter_next(world->cmd_list, &previous);
     }
 
-    if (previous_command && strcmp(command, previous_command) == 0) {
-      /* A duplicate command, it is not added again */
+    if (is_repeated) {
+      /* A duplicate command */
+      /* It is not added again, but the old command moved into position */
+      gtk_list_store_move_after(GTK_LIST_STORE(world->cmd_list),
+                                &previous,
+                                &most_recent);
+      /* most_recent might have become invalid */
+      gtk_tree_model_get_iter_first(world->cmd_list, &most_recent);
+
       if (world->repeat_commands) {
-        /* But in this case, there is already a duplicate entry, so let's
+        /* In this case, a duplicate entry has already been added, so let's
            remove it. */
         gtk_list_store_remove(GTK_LIST_STORE(world->cmd_list), &most_recent);
         --world->cmd_list_size;
@@ -683,8 +650,6 @@ add_recent_command(World *world, const gchar *command)
                            CMDLIST_COL_TENTATIVE, "",
                            -1);
       }
-
-      g_free(previous_command);
     } else {
       /* A new command */
       gtk_list_store_set(GTK_LIST_STORE(world->cmd_list), &most_recent,
