@@ -1,7 +1,7 @@
-/* $Id: kildclient.h,v 1.162 2008/03/23 21:09:42 ekalin Exp $ */
+/* $Id: kildclient.h 1027 2009-02-12 18:27:11Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2008 Eduardo M Kalinowski <ekalin@gmail.com>
+ * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -89,6 +89,42 @@ enum TelnetState_e {
 typedef enum TelnetState_e TelnetState;
 
 
+enum proxy_type_e {
+  PROXY_USE_GLOBAL = -1,
+  PROXY_NONE,
+  PROXY_SOCKS4,
+  PROXY_SOCKS5,
+};
+typedef enum proxy_type_e proxy_type;
+
+enum proxy_socks5_state_e {
+  SOCKS5_STATE_INITIAL,
+  SOCKS5_STATE_AUTH,
+  SOCKS5_STATE_CONNECTION,
+};
+typedef enum proxy_socks5_state_e proxy_socks5_state;
+
+struct proxy_settings_s
+{
+  proxy_type          type;
+  char               *server;
+  char               *port;
+  char               *user;
+  char               *password;
+  proxy_socks5_state  state;
+};
+typedef struct proxy_settings_s proxy_settings;
+
+
+enum NameDisplayStyle_e {
+  NAME_DISPLAY_WORLD,
+  NAME_DISPLAY_WORLD_CHAR,
+  NAME_DISPLAY_CHAR_WORLD,
+  NAME_DISPLAY_CUSTOM,
+};
+typedef enum NameDisplayStyle_e NameDisplayStyle;
+
+
 struct plugin_s
 {
   char     *name;
@@ -300,31 +336,36 @@ typedef struct WorldGUI_s WorldGUI;
 struct world_s
 {
   /* Basic information */
-  gchar           *name;
-  gchar           *new_text_str;
+  gchar           *name;              /* World name */
+  gchar           *display_name;      /* Name displayed in tab & window title */
+  gchar           *new_text_name;     /* Red version of the above */
   gchar           *file;
   gchar           *charset;
   gint             charset_index;
   gchar           *host;
   gchar           *host_ip;
   gchar           *port;
+  proxy_settings   proxy;
+  proxy_settings   proxy_used;
 
   /* Auto-login data */
-  ConnStyle        connection_style;
-  GtkTreeModel    *logon_characters;
-  gint             character_used;
+  ConnStyle            connection_style;
+  GtkTreeModel        *logon_characters;
+  GtkTreeRowReference *character_used;
 
   /* Display settings */
-  char            *terminalfont;
-  char            *entryfont;
-  char            *statusfont;
-  gboolean         usebold;
-  gboolean         wrap;
-  GdkColor        *deffore;
-  GdkColor        *defbold;
-  GdkColor        *defback;
-  GdkColor        *defboldback;
-  GdkColor        *ansicolors;
+  char             *terminalfont;
+  char             *entryfont;
+  char             *statusfont;
+  gboolean          usebold;
+  gboolean          wrap;
+  GdkColor         *deffore;
+  GdkColor         *defbold;
+  GdkColor         *defback;
+  GdkColor         *defboldback;
+  GdkColor         *ansicolors;
+  NameDisplayStyle  name_display_style;
+  char             *name_display_pattern;
 
   /* Scripting support */
   gchar           *scriptfile;
@@ -355,6 +396,7 @@ struct world_s
   /* Input settings */
   gboolean         repeat_commands;
   gboolean         cmd_echo;
+  gboolean         never_hide_input;
   gboolean         store_commands;
   gint             commands_to_save;
   gint             input_n_lines;
@@ -410,7 +452,8 @@ struct world_s
   glong            deleted_lines;
   glong            rawbytes;      /* Bytes received from network */
   glong            bytes;         /* Bytes seen after decompression */
-  struct addrinfo *addrinfo;      /* addrinfo for connecting */
+  struct addrinfo *addrinfo;      /* addrinfo for real connection */
+  guint32          real_ip_for_socks4; /* IP of actual host, for SOCKS4 */
 
   /* Buffers and state */
   TelnetState      telnet_state;             /* State of Telnet IAC FSM */
@@ -544,11 +587,13 @@ struct prefs_s
   gdouble          multi_cmd_delay;
   gint             multi_cmd_group_size;
   GtkPositionType  tab_position;
+  gboolean         hide_single_tab;
   gboolean         urgency_hint;
   char            *browser_command;
   char            *audio_player_command;
   char            *last_open_world;
   gboolean         no_plugin_help_msg;
+  proxy_settings   proxy;
 };
 typedef struct prefs_s Prefs;
 
@@ -632,12 +677,16 @@ typedef struct timer_s Timer;
 
 /* For multi-line send */
 enum MLSendState_e {
-  ML_TRY_INITIALTEXT,
-  ML_INITIALTEXT,
+  ML_TRY_INITIALBUFFER,
+  ML_INITIALBUFFER,
+  ML_TRY_INITIALLINES,
+  ML_INITIALLINES,
   ML_TRY_FILE,
   ML_FILECONTENTS,
-  ML_TRY_FINALTEXT,
-  ML_FINALTEXT,
+  ML_TRY_FINALBUFFER,
+  ML_FINALBUFFER,
+  ML_TRY_FINALLINES,
+  ML_FINALLINES,
   ML_FINISH,
 };
 typedef enum MLSendState_e MLSendState;
@@ -648,6 +697,8 @@ struct MLSendData_s
   MLSendState    state;
   GtkTextBuffer *textInitial;
   GtkTextBuffer *textFinal;
+  GSList        *linesInitial;
+  GSList        *linesFinal;
   gchar         *file;
   FILE          *fp;
   gchar         *textStart;
@@ -655,6 +706,7 @@ struct MLSendData_s
   int            nlines;
   int            linesatime;
   int            currline;
+  GSList        *list_iterator;
   gboolean       dont_close;
 };
 typedef struct MLSendData_s MLSendData;
@@ -715,6 +767,7 @@ enum {
  * Global variables *
  ********************/
 extern GtkWidget *wndMain;              /* The main window widget */
+extern GtkWidget *ntbkWorlds;           /* Notebook with the worlds */
 extern GList     *open_worlds;          /* List of open worlds */
 extern World     *currentWorld;         /* The current world */
 extern World     *world_for_perl;       /* The world for Perl to use */
@@ -825,16 +878,20 @@ gboolean  load_world_from_file(const char *file,
                                gboolean forWorldSelector,
                                GError **error);
 void      load_command_history(World *world);
-World *   get_world_from_name(const gchar *name);
+World    *get_world_from_name(const gchar *name);
 void      save_world_to_file(World *world);
 void      save_command_history(World *world);
 gboolean  edit_world(World     **world,
                      GtkWindow  *parent,
                      gboolean   *newworld,
                      gboolean    modal);
-void      fprintf_escaped(FILE *fp, const char *template, ...);
+void      kc_g_string_append_escaped(GString *str, const char *template, ...);
 gint      disconnected_msgbox(gchar *msg, gboolean can_offline);
 void      mark_as_offline(World *world);
+void      prepare_display_name(World *world);
+gchar    *replace_kc_escapes(const gchar *pattern,
+                             const gchar *name,
+                             const gchar *character);
 
 
 /*********************************************
@@ -1004,7 +1061,7 @@ gboolean  move_trigger(World *world, gint old_pos, gint new_pos);
 void      free_trigger(Trigger *trigger, gpointer data);
 void      list_triggers(World *world, Plugin *plugin);
 void      trigger_precompute_res(World *world);
-void      save_trigger(FILE *fp, Trigger *trigger);
+void      save_trigger(GString *str, Trigger *trigger);
 void      open_test_triggers_dialog(World *world);
 
 
@@ -1017,7 +1074,7 @@ gboolean move_alias(World *world, gint old_pos, gint new_pos);
 void     free_alias(Alias *alias, gpointer data);
 void     list_aliases(World *world, Plugin *plugin);
 void     alias_precompute_res(World *world);
-void     save_alias(FILE *fp, Alias *alias);
+void     save_alias(GString *str, Alias *alias);
 
 
 /********************************
@@ -1028,7 +1085,7 @@ void     remove_macro(World *world, GSList *macroitem);
 gboolean move_macro(World *world, gint old_pos, gint new_pos);
 void     free_macro(Macro *macro, gpointer data);
 void     list_macros(World *world, Plugin *plugin);
-void     save_macro(FILE *fp, Macro *macro);
+void     save_macro(GString *str, Macro *macro);
 
 
 /********************************
@@ -1040,7 +1097,7 @@ void     remove_timer(World *world, GSList *timeritem);
 gboolean move_timer(World *world, gint old_pos, gint new_pos);
 void     free_timer(Timer *timer, gpointer data);
 void     list_timers(World *world, Plugin *plugin);
-void     save_timer(FILE *fp, Timer *timer);
+void     save_timer(GString *str, Timer *timer);
 
 
 /*********************************************
@@ -1070,8 +1127,8 @@ gboolean      delete_hook(World *world, const gchar *hookname, int pos);
 gboolean      move_hook(World *world, const gchar *hookname,
                         gint old_pos, gint new_pos);
 void          free_hook(Hook *hook, gpointer data);
-void          save_hooks(FILE *fp, World *world);
-void          save_hook(FILE *fp, Hook *hook);
+void          save_hooks(GString *str, World *world);
+void          save_hook(GString *str, Hook *hook);
 GSList      **get_hook_list_for_writing(World        *world,
                                         const gchar  *hookname,
                                         gint        **poscounter);
@@ -1084,8 +1141,8 @@ GtkTreeModel *get_hook_model(World *world, const gchar *hookname);
 /*********************************
  * Function Prototypes - Logging *
  *********************************/
-void     write_log_line(World *world, const gchar *line);
-gboolean start_log(World *world, gchar **errormsg);
+void     write_log_line(World *world, const gchar *line, time_t *line_time);
+gboolean start_log(World *world, gint lines, gchar **errormsg);
 void     stop_log(World *world);
 
 
