@@ -1,7 +1,7 @@
-/* $Id: main.c,v 1.52 2008/05/03 13:13:53 ekalin Exp $ */
+/* $Id: main.c 1014 2009-02-01 12:46:52Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2008 Eduardo M Kalinowski <ekalin@gmail.com>
+ * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -127,7 +127,7 @@ main(int argc, char *argv[], char *env[])
   g_option_context_free(opt_context);
   if (wants_version) {
     printf(PACKAGE_NAME " " PACKAGE_VERSION "\n");
-    printf(_("Written by Eduardo M Kalinowski (ekalin@gmail.com)\n"));
+    printf(_("Written by Eduardo M Kalinowski (eduardo@kalinowski.com.br)\n"));
     exit(0);
   }
   if (dumpfile) {
