@@ -1,7 +1,7 @@
-/* $Id: mainwindow.c,v 1.136 2008/04/05 13:38:17 ekalin Exp $ */
+/* $Id: mainwindow.c 1017 2009-02-02 18:34:39Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2008 Eduardo M Kalinowski <ekalin@gmail.com>
+ * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -55,18 +55,13 @@
  ******************/
 World     *currentWorld         = NULL;
 GtkWidget *wndMain              = NULL;
+GtkWidget *ntbkWorlds           = NULL;
 GList     *open_worlds          = NULL;
 gboolean   window_has_focus     = TRUE;
 gint       worlds_with_new_text = 0;
 gboolean   debug_matches        = 0;
 
 
-/*************************
- * File global variables *
- *************************/
-static GtkWidget *ntbkWorlds;
-
-
 /***********************
  * Function prototypes *
  ***********************/
@@ -247,8 +242,11 @@ window_focus_in_cb(GtkWidget     *widget,
 
   currentWorld->has_focus = TRUE;
   if (currentWorld->has_unread_text) {
-    gtk_label_set_text(currentWorld->gui->lblNotebook,
-                       currentWorld->name);
+    /* If in offline mode, then the tab is greyed. Let's not change it. */
+    if (currentWorld->connected) {
+      gtk_label_set_text(currentWorld->gui->lblNotebook,
+                         currentWorld->display_name);
+    }
     currentWorld->has_unread_text = FALSE;
     --worlds_with_new_text;
   }
@@ -306,9 +304,9 @@ adjust_window_title(void)
 
   if (currentWorld) {
     if (currentWorld->connected) {
-      worldname = g_strdup_printf(" - %s", currentWorld->name);
+      worldname = g_strdup_printf(" - %s", currentWorld->display_name);
     } else {
-      worldname = g_strdup_printf(" - [%s]", currentWorld->name);
+      worldname = g_strdup_printf(" - [%s]", currentWorld->display_name);
     }
   } else {
     worldname = g_strdup("");
@@ -405,7 +403,7 @@ update_world_with_focus(gpointer data)
     currentWorld->is_current_tab = TRUE;
     if (currentWorld->has_unread_text) {
       gtk_label_set_text(currentWorld->gui->lblNotebook,
-                         currentWorld->name);
+                         currentWorld->display_name);
       currentWorld->has_unread_text = FALSE;
       --worlds_with_new_text;
     }
@@ -511,7 +509,8 @@ edit_this_world_cb(GtkWidget *widget, gpointer data)
 
   edit_world(&currentWorld, GTK_WINDOW(wndMain), &dummy, FALSE);
 
-  gtk_label_set_text(currentWorld->gui->lblNotebook, currentWorld->name);
+  gtk_label_set_text(currentWorld->gui->lblNotebook,
+                     currentWorld->display_name);
 }
 
 
@@ -523,6 +522,9 @@ menu_statistics_cb(GtkWidget *widget, gpointer data)
   GtkWidget *lblName;
   GtkWidget *lblHost;
   GtkWidget *lblPort;
+  GtkWidget *lblProxyType;
+  GtkWidget *lblProxyHost;
+  GtkWidget *lblProxyPort;
   GtkWidget *lblConnTime;
   GtkWidget *lblIdleTime;
   GtkWidget *lblCompression;
@@ -552,7 +554,7 @@ menu_statistics_cb(GtkWidget *widget, gpointer data)
 
   /* Basic information */
   gtk_label_set_text(GTK_LABEL(lblName), currentWorld->name);
-  if (currentWorld->host_ip) {
+  if (currentWorld->proxy_used.type == PROXY_NONE && currentWorld->host_ip) {
     str = g_strdup_printf("%s (%s)", currentWorld->host, currentWorld->host_ip);
   } else {
     str = g_strdup(currentWorld->host);
@@ -561,6 +563,22 @@ menu_statistics_cb(GtkWidget *widget, gpointer data)
   g_free(str);
   gtk_label_set_text(GTK_LABEL(lblPort), currentWorld->port);
 
+  /* Proxy information */
+  if (currentWorld->proxy_used.type != PROXY_NONE) {
+    lblProxyType = glade_xml_get_widget(gladexml, "lblProxyType");
+    lblProxyHost = glade_xml_get_widget(gladexml, "lblProxyHost");
+    lblProxyPort = glade_xml_get_widget(gladexml, "lblProxyPort");
+
+    if (currentWorld->proxy_used.type == PROXY_SOCKS4) {
+      gtk_label_set_text(GTK_LABEL(lblProxyType), "SOCKS4");
+    } else if (currentWorld->proxy_used.type == PROXY_SOCKS5) {
+      gtk_label_set_text(GTK_LABEL(lblProxyType), "SOCKS5");
+    }
+
+    gtk_label_set_text(GTK_LABEL(lblProxyHost), currentWorld->proxy_used.server);
+    gtk_label_set_text(GTK_LABEL(lblProxyPort), currentWorld->proxy_used.port);
+  }
+
   if (currentWorld->connected) {
     now = time(NULL);
     timestr = g_string_new(NULL);
@@ -812,6 +830,11 @@ attach_world_gui(WorldGUI *gui)
                                    gui->vbox,
                                    TRUE);
 
+  if (globalPrefs.hide_single_tab) {
+    gtk_notebook_set_show_tabs(GTK_NOTEBOOK(ntbkWorlds),
+                               gtk_notebook_get_n_pages(GTK_NOTEBOOK(ntbkWorlds)) != 1);
+  }
+
   gtk_widget_show_all(ntbkWorlds);
 
   return page;
@@ -828,6 +851,11 @@ detach_world_gui(WorldGUI *gui)
   old_page = gtk_notebook_page_num(GTK_NOTEBOOK(ntbkWorlds), gui->vbox);
   gtk_notebook_remove_page(GTK_NOTEBOOK(ntbkWorlds), old_page);
 
+  if (globalPrefs.hide_single_tab) {
+    gtk_notebook_set_show_tabs(GTK_NOTEBOOK(ntbkWorlds),
+                               gtk_notebook_get_n_pages(GTK_NOTEBOOK(ntbkWorlds)) != 1);
+  }
+
   new_page = gtk_notebook_get_current_page(GTK_NOTEBOOK(ntbkWorlds));
   update_world_with_focus(GINT_TO_POINTER(new_page));
 }
