@@ -1,7 +1,7 @@
-/* $Id: net.c,v 1.107 2008/03/23 21:09:37 ekalin Exp $ */
+/* $Id: net.c 1024 2009-02-09 18:01:06Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2008 Eduardo M Kalinowski <ekalin@gmail.com>
+ * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -50,6 +50,7 @@
 
 #include "kildclient.h"
 #include "perlscript.h"
+#include "net.h"
 #include "ansi.h"
 
 
@@ -66,14 +67,6 @@ const int cert_type_priority[3] = { GNUTLS_CRT_X509, GNUTLS_CRT_OPENPGP, 0 };
 /**************
  * Data types *
  **************/
-struct resolve_data_s
-{
-  World           *world;
-  gchar           *host;
-  gchar           *port;
-  struct addrinfo *addrinfo;
-  int              ret;
-};
 
 
 /***************************
@@ -102,12 +95,10 @@ struct resolve_data_s
 /***********************
  * Function prototypes *
  ***********************/
-static gpointer resolve_name_thread(gpointer data);
-static gboolean continue_connecting_cb(gpointer data);
 static gboolean connect_ready_cb(GIOChannel   *iochan,
                                  GIOCondition  cond,
                                  gpointer      data);
-static gboolean connected_to(World *world);
+static gboolean connected_to_dispatcher(World *world);
 static void     disconnected_decide_action(World *world, gint action);
 static gboolean data_ready_cb(GIOChannel   *iochan,
                               GIOCondition  cond,
@@ -120,6 +111,7 @@ static void     process_telnet_iac(World *world, int pos, int end);
 static void     start_mccp(World *world, int pos, int end);
 static void     end_mccp(World *world);
 static ssize_t  write_escaped(World *world, const void *buf, size_t count);
+
 /* Send and receive data */
 #ifdef HAVE_LIBGNUTLS
 static ssize_t kc_recv(World *world, void *buf, size_t s);
@@ -190,8 +182,9 @@ connect_to(World *world, gchar **errmsg)
   /* Set up gui */
   if (!world->connected_at_least_once) {
     configure_gui(world->gui, world);
+    prepare_display_name(world);
   }
-  gtk_label_set_text(world->gui->lblNotebook, world->name);
+  gtk_label_set_text(world->gui->lblNotebook, world->display_name);
 
   if (g_list_index(open_worlds, world) == -1) {
     open_worlds = g_list_append(open_worlds, world);
@@ -232,7 +225,21 @@ connect_to(World *world, gchar **errmsg)
 
   world->connected_at_least_once = TRUE;
 
-  msg = g_strdup_printf(_("Resolving host %s..."), world->host);
+  proxy_info_free(&world->proxy_used);
+  if (world->proxy.type == PROXY_USE_GLOBAL) {
+    proxy_info_copy(&globalPrefs.proxy, &world->proxy_used);
+  } else if (world->proxy.type == PROXY_NONE) {
+    world->proxy_used.type = PROXY_NONE;
+  } else {
+    proxy_info_copy(&world->proxy, &world->proxy_used);
+  }
+
+  if (world->proxy_used.type == PROXY_SOCKS5) {
+    msg = g_strdup_printf(_("Resolving proxy host %s..."),
+                          world->proxy_used.server);
+  } else {
+    msg = g_strdup_printf(_("Resolving host %s..."), world->host);
+  }
   gtk_label_set_text(world->gui->lblStatus, msg);
   ansitextview_append_string_with_fgcolor(world->gui, msg,
                                           globalPrefs.idxInfoMsgColor);
@@ -243,9 +250,23 @@ connect_to(World *world, gchar **errmsg)
 
   /* Spawn a thread to resolve the host name */
   resolve_data = g_new(struct resolve_data_s, 1);
-  resolve_data->world = world;
-  resolve_data->host  = g_strdup(world->host);
-  resolve_data->port  = g_strdup(world->port);
+  resolve_data->world    = world;
+  if (world->proxy_used.type == PROXY_SOCKS4) {
+    resolve_data->host      = g_strdup(world->host);
+    resolve_data->port      = g_strdup(world->port);
+    resolve_data->callback  = proxy_socks4_resolved_real_host_cb;
+    resolve_data->ai_family = AF_INET;
+  } else if (world->proxy_used.type == PROXY_SOCKS5) {
+    resolve_data->host     = g_strdup(world->proxy_used.server);
+    resolve_data->port     = g_strdup(world->proxy_used.port);
+    resolve_data->callback = continue_connecting_cb;
+    resolve_data->ai_family = AF_UNSPEC;
+  } else {
+    resolve_data->host     = g_strdup(world->host);
+    resolve_data->port     = g_strdup(world->port);
+    resolve_data->callback = continue_connecting_cb;
+    resolve_data->ai_family = AF_UNSPEC;
+  }
   if (!g_thread_create(resolve_name_thread, resolve_data, FALSE, NULL)) {
     *errmsg = g_strdup(_("Could not start name-resolution thread."));
     return FALSE;
@@ -255,7 +276,6 @@ connect_to(World *world, gchar **errmsg)
 }
 
 
-static
 gpointer
 resolve_name_thread(gpointer data)
 {
@@ -263,7 +283,7 @@ resolve_name_thread(gpointer data)
   struct addrinfo  hints;
 
   memset(&hints, 0, sizeof(struct addrinfo));
-  hints.ai_family   = AF_UNSPEC;
+  hints.ai_family   = resolve_data->ai_family;
   hints.ai_socktype = SOCK_STREAM;
   hints.ai_protocol = 0;
   hints.ai_flags    = 0;
@@ -271,13 +291,12 @@ resolve_name_thread(gpointer data)
   resolve_data->ret = getaddrinfo(resolve_data->host, resolve_data->port,
                                   &hints,
                                   &resolve_data->addrinfo);
-  g_idle_add(continue_connecting_cb, resolve_data);
+  g_idle_add(resolve_data->callback, resolve_data);
 
   return NULL;
 }
 
 
-static
 gboolean
 continue_connecting_cb(gpointer data)
 {
@@ -291,7 +310,6 @@ continue_connecting_cb(gpointer data)
   u_long flags;
 #endif
 
-  g_free(resolve_data->host);
   g_free(resolve_data->port);
 
   if (resolve_data->ret != 0) {
@@ -303,10 +321,11 @@ continue_connecting_cb(gpointer data)
       msg = g_locale_to_utf8(gai_strerror(resolve_data->ret), -1,
                              NULL, &len, NULL);
       world->errmsg = g_strdup_printf(_("Could not resolve host %s: %s"),
-                                      world->host, msg);
+                                      resolve_data->host, msg);
       g_free(msg);
       disconnect_world(world);
     }
+    g_free(resolve_data->host);
     g_free(resolve_data);
 
     /* Only run once */
@@ -314,6 +333,7 @@ continue_connecting_cb(gpointer data)
   }
 
   world->addrinfo = resolve_data->addrinfo;
+  g_free(resolve_data->host);
   g_free(resolve_data);
 
   /* Create IP address representation */
@@ -327,9 +347,15 @@ continue_connecting_cb(gpointer data)
                 NI_NUMERICHOST);
   }
 
-  msg = g_strdup_printf(_("Attempting to connect%s to host %s (%s) port %s..."),
-                        world->use_tls ? _(" with SSL") : "",
-                        world->host, world->host_ip, world->port);
+  if (world->proxy_used.type == PROXY_NONE) {
+    msg = g_strdup_printf(_("Attempting to connect to host %s (%s) port %s..."),
+                          world->host, world->host_ip, world->port);
+  } else {
+    msg = g_strdup_printf(_("Attempting to connect to proxy host %s (%s) port %s..."),
+                          world->proxy_used.server,
+                          world->host_ip,
+                          world->proxy_used.port);
+  }
   ansitextview_append_string_with_fgcolor(world->gui, msg,
                                           globalPrefs.idxInfoMsgColor);
   ansitextview_append_string(world->gui, "\n");
@@ -399,7 +425,7 @@ continue_connecting_cb(gpointer data)
   g_io_channel_set_encoding(world->iochan, NULL, NULL);
 
   if (!world->connecting) {
-    connected_to(world);
+    connected_to_dispatcher(world);
   }
 
   /* This is only called once. */
@@ -426,13 +452,19 @@ connect_ready_cb(GIOChannel   *iochan,
       return TRUE;  /* Still connecting. */
     }
 
-    world->errmsg = g_strdup_printf(_("Unable to connect to host %s: %s."),
-                                    world->host, g_strerror(errno));
+    if (world->proxy_used.type == PROXY_NONE) {
+      world->errmsg = g_strdup_printf(_("Unable to connect to host %s: %s."),
+                                      world->host, g_strerror(errno));
+    } else {
+      world->errmsg = g_strdup_printf(_("Unable to connect to proxy host %s: %s."),
+                                      world->proxy_used.server,
+                                      g_strerror(errno));
+    }
 
     disconnect_world(world);
     return FALSE;
   } else {
-    connected_to(world);
+    connected_to_dispatcher(world);
     return FALSE;
   }
 }
@@ -455,7 +487,7 @@ connect_ready_cb(GIOChannel   *iochan,
                 world->addrinfo->ai_addr, world->addrinfo->ai_addrlen);
   error = WSAGetLastError();
   if (error == WSAEISCONN) { /* Connection succeeded */
-    connected_to(world);
+    connected_to_dispatcher(world);
     return FALSE;
   }
 
@@ -464,8 +496,13 @@ connect_ready_cb(GIOChannel   *iochan,
     sockerror = WSAGetLastError();
   }
   if (sockerror) {
-    world->errmsg = g_strdup_printf(_("Unable to connect to host %s: Error #%d."),
-                                    world->host, sockerror);
+    if (world->proxy_used.type == PROXY_NONE) {
+      world->errmsg = g_strdup_printf(_("Unable to connect to host %s: Error #%d."),
+                                      world->host, sockerror);
+    } else {
+      world->errmsg = g_strdup_printf(_("Unable to connect to proxy host %s: Error #%d."),
+                                      world->proxy_used.server, sockerror);
+    }
     disconnect_world(world);
     return FALSE;
   }
@@ -477,8 +514,13 @@ connect_ready_cb(GIOChannel   *iochan,
   /* This should never be reached. But under Windows, it is better
      to be safe. */
   g_warning("Unreachable code in connect_read_cb.");
-  world->errmsg = g_strdup_printf(_("Unable to connect to host %s: Error #%d."),
-                                  world->host, sockerror);
+  if (world->proxy_used.type == PROXY_NONE) {
+    world->errmsg = g_strdup_printf(_("Unable to connect to host %s: Error #%d."),
+                                    world->host, sockerror);
+  } else {
+    world->errmsg = g_strdup_printf(_("Unable to connect to proxy host %s: Error #%d."),
+                                    world->proxy_used.server, sockerror);
+  }
   disconnect_world(world);
   return FALSE;
 }
@@ -486,20 +528,46 @@ connect_ready_cb(GIOChannel   *iochan,
 
 
 static
+gboolean
+connected_to_dispatcher(World *world)
+{
+  /* When this function is called, a connection to the proxy host or
+     the real host (if no proxy is being used) has been established.
+     This function then calls the appropriate action if a proxy is
+     being used. */
+  if (world->proxy_used.type == PROXY_SOCKS4) {
+    return proxy_socks4_connected_to(world);
+  } else if (world->proxy_used.type == PROXY_SOCKS5) {
+    return proxy_socks5_connected_to(world);
+  } else {
+    return connected_to(world);
+  }
+}
+
+
 gboolean
 connected_to(World *world)
 {
+  /* A connection has been established to the real mud host, via proxy
+     or indirectly. */
   gchar *status_text;
 
 #ifdef HAVE_LIBGNUTLS
   if (world->use_tls) {
 #ifndef __MINGW32__
-  int    flags;
+    int    flags;
 #else /* defined __MINGW32__ */
-  u_long flags;
+    u_long flags;
 #endif
     int ret;
 
+    status_text = g_strdup_printf(_("Performing SSL handshake..."));
+    ansitextview_append_string_with_fgcolor(world->gui, status_text,
+                                            globalPrefs.idxInfoMsgColor);
+    ansitextview_append_string(world->gui, "\n");
+    gtk_label_set_text(world->gui->lblStatus, status_text);
+    g_free(status_text);
+
 #ifndef __MINGW32__
     flags = fcntl(world->sock, F_GETFL);
     flags &= ~O_NONBLOCK;
@@ -531,9 +599,20 @@ connected_to(World *world)
   world->connected_time    = time(NULL);
   world->last_command_time = time(NULL);
 
-  status_text = g_strdup_printf(_("Connected%s to host %s (%s) port %s."),
-                                world->use_tls ? _(" with SSL") : "",
-                                world->host, world->host_ip, world->port);
+  if (world->proxy_used.type == PROXY_NONE) {
+    status_text = g_strdup_printf(_("Connected to host %s (%s) port %s."),
+                                  world->host, world->host_ip, world->port);
+  } else {
+    status_text = g_strdup_printf(_("Connected to host %s port %s, via proxy %s (%s) port %s."),
+                                  world->host, world->port,
+                                  world->proxy_used.server, world->host_ip,
+                                  world->proxy_used.port);
+  }
+  if (world->use_tls) {
+    gchar *tmp = status_text;
+    status_text = g_strconcat(tmp, _(" SSL enabled."), NULL);
+    g_free(tmp);
+  }
   ansitextview_append_string_with_fgcolor(world->gui, status_text,
                                           globalPrefs.idxInfoMsgColor);
   ansitextview_append_string(world->gui, "\n");
@@ -547,7 +626,7 @@ connected_to(World *world)
     gchar *errmsg = NULL;
 
     stop_log(world);
-    if (start_log(world, &errmsg)) {
+    if (start_log(world, 0, &errmsg)) {
       ansitextview_append_stringf(world->gui,
                                   _("Logging to file %s.\n"),
                                   world->log_actual_file);
@@ -974,7 +1053,7 @@ process_recv_line(World *world, int start, int end, gboolean incomplete)
       if (!world->has_focus) {
         if (!world->has_unread_text) {
           world->has_unread_text = TRUE;
-          gtk_label_set_markup(world->gui->lblNotebook, world->new_text_str);
+          gtk_label_set_markup(world->gui->lblNotebook, world->new_text_name);
           if (worlds_with_new_text != -1) {
             ++worlds_with_new_text;
 
@@ -1024,13 +1103,20 @@ process_telnet_iac(World *world, int pos, int end)
         world->telnet_state = TELNETFSM_DATA_CR;
         break;
 
+      case '\b': /* Backspace */
+        if (world->pbuf_size > 0
+            && world->pbuffer[world->pbuf_size - 1] != '\n') {
+          --world->pbuf_size;
+        }
+        break;
+
       default:
         world->pbuffer[world->pbuf_size++] = world->dbuffer[pos];
       }
       break;
 
     case TELNETFSM_DATA_NL:
-      if (world->dbuffer[pos] != '\r') {
+      if (world->dbuffer[pos] != '\r' && world->dbuffer[pos] != '\b') {
         world->pbuffer[world->pbuf_size++] = world->dbuffer[pos];
         if (world->dbuffer[pos] != '\n') {
           world->telnet_state = TELNETFSM_DATA;
@@ -1131,7 +1217,9 @@ process_telnet_iac(World *world, int pos, int end)
       switch (world->dbuffer[pos]) {
       case TELNET_OPT_ECHO:
         world->noecho = TRUE;
-        simo_combo_box_set_visibility(world->gui->cmbEntry, FALSE);
+        if (!world->never_hide_input) {
+          simo_combo_box_set_visibility(world->gui->cmbEntry, FALSE);
+        }
         iac_response[0] = TELNET_IAC;
         iac_response[1] = TELNET_DO;
         iac_response[2] = TELNET_OPT_ECHO;
@@ -1268,10 +1356,22 @@ start_mccp(World *world, int pos, int end)
       world->zstream->next_out  = world->dbuffer;
       world->zstream->avail_out = MAX_BUFFER;
 
-      status_text = g_strdup_printf(_("Connected%s to host %s (%s) port %s, MCCP version %d enabled."),
-                                    world->use_tls ? _(" with SSL") : "",
-                                    world->host, world->host_ip, world->port,
-                                    world->mccp_ver);
+      if (world->proxy_used.type == PROXY_NONE) {
+        status_text = g_strdup_printf(_("Connected to host %s (%s) port %s, MCCP version %d enabled."),
+                                      world->host, world->host_ip, world->port,
+                                      world->mccp_ver);
+      } else {
+        status_text = g_strdup_printf(_("Connected to host %s port %s, via proxy %s (%s) port %s, MCCP version %d enabled."),
+                                      world->host, world->port,
+                                      world->proxy_used.server, world->host_ip,
+                                      world->proxy_used.port,
+                                      world->mccp_ver);
+      }
+      if (world->use_tls) {
+        gchar *tmp = status_text;
+        status_text = g_strconcat(tmp, _(" SSL enabled."), NULL);
+        g_free(tmp);
+      }
       gtk_label_set_text(world->gui->lblStatus, status_text);
       g_free(status_text);
     } else {
@@ -1293,9 +1393,20 @@ end_mccp(World *world)
   g_free(world->zstream);
   world->zstream = NULL;
 
-  status_text = g_strdup_printf(_("Connected%s to host %s (%s) port %s."),
-                                world->use_tls ? _(" with SSL") : "",
-                                world->host, world->host_ip, world->port);
+  if (world->proxy_used.type == PROXY_NONE) {
+    status_text = g_strdup_printf(_("Connected to host %s (%s) port %s."),
+                                  world->host, world->host_ip, world->port);
+  } else {
+    status_text = g_strdup_printf(_("Connected to host %s port %s, via proxy %s (%s) port %s."),
+                                  world->host, world->port,
+                                  world->proxy_used.server, world->host_ip,
+                                  world->proxy_used.port);
+  }
+  if (world->use_tls) {
+    gchar *tmp = status_text;
+    status_text = g_strconcat(tmp, _(" SSL enabled."), NULL);
+    g_free(tmp);
+  }
   gtk_label_set_text(world->gui->lblStatus, status_text);
   g_free(status_text);
 }
@@ -1425,15 +1536,22 @@ perform_auto_logon(World *world)
   gchar       *character = NULL;
   gchar       *password;
   gchar       *command = NULL;
+  GtkTreePath *path;
   GtkTreeIter  iter;
 
-  if (world->character_used == -1 || world->connection_style == NONE) {
+  if (world->character_used == NULL || world->connection_style == NONE) {
     return;
   }
 
-  gtk_tree_model_iter_nth_child(world->logon_characters,
-                                &iter, NULL,
-                                world->character_used);
+  path = gtk_tree_row_reference_get_path(world->character_used);
+  if (!path) {
+    gtk_tree_row_reference_free(world->character_used);
+    world->character_used = NULL;
+    return;
+  }
+
+  gtk_tree_model_get_iter(world->logon_characters,
+                          &iter, path);
   gtk_tree_model_get(world->logon_characters, &iter,
                      LOGON_CHAR, &character,
                      LOGON_PASS, &password,
@@ -1457,6 +1575,7 @@ perform_auto_logon(World *world)
   g_free(command);
   g_free(character);
   g_free(password);
+  gtk_tree_path_free(path);
 }
 
 
