@@ -1,7 +1,7 @@
-/* $Id: perlscript.c,v 1.128 2008/03/23 21:09:36 ekalin Exp $ */
+/* $Id: perlscript.c 1033 2009-02-19 21:34:07Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2008 Eduardo M Kalinowski <ekalin@gmail.com>
+ * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -57,6 +57,7 @@
 /***********************
  * Function prototypes *
  ***********************/
+static GSList *AVtoGSList(AV *array);
 static SV     *create_world_var(World *world);
 static SV     *create_window_var(void);
 static World  *fetch_world(int items, SV *self_ref, HV **self_to_store);
@@ -89,7 +90,7 @@ static char *my_argv_static[] = { "kildperl",
 void
 init_perl_system(void)
 {
-  PERL_SYS_INIT3(&my_argc_static, &my_argv_static, NULL);
+  PERL_SYS_INIT(&my_argc_static, &my_argv_static);
 }
 
 
@@ -176,7 +177,7 @@ create_world_var(World *world)
 
   worldptriv = PTR2IV(world);
   worldptrsv = newSViv(worldptriv);
-  hv_store(world_hash, "WORLDPTR", 8, worldptrsv, 0);
+  (void) hv_store(world_hash, "WORLDPTR", 8, worldptrsv, 0);
 
   world_ref = newRV_noinc((SV *) world_hash);
   world_pkg_stash = gv_stashpv("World", TRUE);
@@ -357,7 +358,9 @@ precompute_re(PerlInterpreter *interpreter,
   old_interpreter = PERL_GET_CONTEXT;
   PERL_SET_CONTEXT(interpreter);
 
-  command = newSVpvf("$_re = qr<%s>%s;", re_string, ignore_case ? "i" : "");
+  command = newSVpvf("$_re = qr\x05%s\x05%s;",
+                     re_string,
+                     ignore_case ? "i" : "");
   ENTER;
   SAVETMPS;
   PUSHMARK(SP);
@@ -377,6 +380,28 @@ precompute_re(PerlInterpreter *interpreter,
 }
 
 
+static
+GSList *
+AVtoGSList(AV *array)
+{
+  GSList *list = NULL;
+  int     i;
+  int     max;
+  SV    **element;
+
+  /* max is the highest index number of the array */
+  max = av_len(array);
+  for (i = 0; i <= max; ++i) {
+    element = av_fetch(array, i, FALSE);
+    if (element) {
+      list = g_slist_append(list, g_strdup(SvPV_nolen(*element)));
+    }
+  }
+
+  return list;
+}
+
+
 void
 menu_perl_run_cb(GtkWidget *widget, gpointer data)
 {
@@ -760,6 +785,7 @@ static
 void
 XS_getcharacter(pTHX_ CV *perlcv)
 {
+  GtkTreePath *path;
   GtkTreeIter  iter;
   gchar       *character = NULL;
   FETCHWORLDVARS;
@@ -767,18 +793,25 @@ XS_getcharacter(pTHX_ CV *perlcv)
 
   FETCHWORLD;
 
-  if (world->character_used == -1 || world->connection_style == NONE) {
+  if (world->character_used == NULL || world->connection_style == NONE) {
     XSRETURN_UNDEF;
   }
 
-  gtk_tree_model_iter_nth_child(world->logon_characters,
-                                &iter, NULL,
-                                world->character_used);
+  path = gtk_tree_row_reference_get_path(world->character_used);
+  if (!path) {
+    gtk_tree_row_reference_free(world->character_used);
+    world->character_used = NULL;
+    XSRETURN_UNDEF;
+  }
+
+  gtk_tree_model_get_iter(world->logon_characters,
+                          &iter, path);
   gtk_tree_model_get(world->logon_characters, &iter,
                      LOGON_CHAR, &character,
                      -1);
   XST_mPV(0, character);
   g_free(character);
+  gtk_tree_path_free(path);
 
   XSRETURN(1);
 }
@@ -916,44 +949,197 @@ XS_send(pTHX_ CV *perlcv)
 
 static
 void
-XS_sendfile(pTHX_ CV *perlcv)
+XS__mlsend(pTHX_ CV *perlcv)
 {
-  MLSendData *mlcontext;
+  HV         *attributes;
+  SV        **value;
+  AV         *AVinitialtext = NULL;
+  AV         *AVfinaltext   = NULL;
+  char       *file          = NULL;
   gdouble     delay;
+  MLSendData *mlcontext;
   FETCHWORLDVARS;
   dXSARGS;
 
   FETCHWORLD;
 
-  if (items == 1) {
-    warn(_("%s: Too few arguments"), "sendfile");
+  /* Check argument */
+  if (items < 2) {
+    warn(_("%s: This is an internal function, do not call directly."), "_mlsend");
+    XSRETURN_UNDEF;
+  }
+  if (!SvROK(ST(1)) || SvTYPE(SvRV(ST(1))) != SVt_PVHV) {
+    warn(_("%s: Wrong argument type"), "_mlsend");
     XSRETURN_EMPTY;
   }
 
-  if (items >= 3) {
-    delay = SvNV(ST(2));
+  /* Fetch attributes */
+  attributes = (HV *) SvRV(ST(1));
+
+  /* See if everthing is alright before starting to allocate memory */
+  value = hv_fetch(attributes, "initialtext", 11, FALSE);
+  if (value) {
+    if (!SvROK(*value) || SvTYPE(SvRV(*value)) != SVt_PVAV) {
+      warn(_("%s: This is an internal function, do not call directly."), "_mlsend");
+      XSRETURN_EMPTY;
+    } else {
+      AVinitialtext = (AV *) SvRV(*value);
+    }
+  }
+
+  value = hv_fetch(attributes, "finaltext", 9, FALSE);
+  if (value) {
+    if (!SvROK(*value) || SvTYPE(SvRV(*value)) != SVt_PVAV) {
+      warn(_("%s: This is an internal function, do not call directly."), "_mlsend");
+      XSRETURN_EMPTY;
+    } else {
+      AVfinaltext = (AV *) SvRV(*value);
+    }
+  }
+
+  value = hv_fetch(attributes, "file", 4, FALSE);
+  if (value) {
+    FILE *fp;
+
+    file = SvPV_nolen(*value);
+    /* See if file can be read */
+    if ((fp = fopen(file, "r")) == NULL) {
+      XSRETURN_NO;
+    }
+    fclose(fp);
+  }
+
+  /* No problems - let's prepare the structure for ML Send. */
+  mlcontext = calloc(1, sizeof(MLSendData));
+  mlcontext->state = ML_TRY_INITIALLINES;
+  mlcontext->world = world;
+
+  if (AVinitialtext) {
+    mlcontext->linesInitial = AVtoGSList(AVinitialtext);
+  }
+  if (AVfinaltext) {
+    mlcontext->linesFinal = AVtoGSList(AVfinaltext);
+  }
+  if (file) {
+    mlcontext->file = g_strdup(file);
+  }
+
+  /* Fetch other attributes */
+  value = hv_fetch(attributes, "linestart", 9, FALSE);
+  if (value) {
+    mlcontext->textStart = g_strdup(SvPV_nolen(*value));
+  } else {
+    mlcontext->textStart = g_strdup("");
+  }
+
+  value = hv_fetch(attributes, "lineend", 7, FALSE);
+  if (value) {
+    mlcontext->textEnd = g_strdup(SvPV_nolen(*value));
+  } else {
+    mlcontext->textEnd = g_strdup("");
+  }
+
+  value = hv_fetch(attributes, "delay", 5, FALSE);
+  if (value) {
+    delay = SvNV(*value);
   } else {
     delay = globalPrefs.multi_cmd_delay;
   }
 
-  mlcontext = calloc(1, sizeof(MLSendData));
-  mlcontext->state      = ML_TRY_FILE;
-  mlcontext->world      = world;
-  mlcontext->file       = g_strdup(SvPV_nolen(ST(1)));
-  mlcontext->textStart  = g_strdup("");
-  mlcontext->textEnd    = g_strdup("");
-  if (items == 4) {
-    mlcontext->linesatime = SvIV(ST(3));
+  value = hv_fetch(attributes, "linesatime", 10, FALSE);
+  if (value) {
+    mlcontext->linesatime = SvIV(*value);
   } else {
     mlcontext->linesatime = globalPrefs.multi_cmd_group_size;
   }
 
   do_multi_line_send(mlcontext, delay);
 
+  XSRETURN_YES;
+}
+
+
+static
+void
+XS_interpret(pTHX_ CV *perlcv)
+{
+  char   *line;
+  STRLEN  len;
+  FETCHWORLDVARS;
+  dXSARGS;
+
+  FETCHWORLD;
+
+  if (items == 1) {
+    warn(_("%s: Too few arguments"), "interpret");
+    XSRETURN_EMPTY;
+  }
+
+  line = SvPV(ST(1), len);
+  parse_commands(world, line, len);
+
   XSRETURN_EMPTY;
 }
 
 
+static
+void
+XS_getline(pTHX_ CV *perlcv)
+{
+  int          line;
+  int          last_line;
+  GtkTextIter  start_iter;
+  GtkTextIter  end_iter;
+  gchar       *text;
+  FETCHWORLDVARS;
+  dXSARGS;
+
+  FETCHWORLD;
+
+  if (items == 1) {
+    warn(_("%s: Too few arguments"), "getline");
+    XSRETURN_EMPTY;
+  }
+
+  line = SvIV(ST(1));
+  last_line = gtk_text_buffer_get_line_count(world->gui->txtBuffer);
+  if (line < 0) {
+    /* Negative: nth line from the bottom */
+    line = last_line - (-line) - 1;
+  } else {
+    /* Positive: line number */
+    line = line - world->deleted_lines - 1;
+
+    if (line > last_line) {
+      XSRETURN_EMPTY;
+    }
+  }
+
+  if (line < 0) {
+    XSRETURN_EMPTY;
+  }
+
+  gtk_text_buffer_get_iter_at_line(world->gui->txtBuffer,
+                                   &start_iter, line);
+  end_iter = start_iter;
+  gtk_text_iter_forward_to_line_end(&end_iter);
+  text = gtk_text_buffer_get_text(world->gui->txtBuffer,
+                                  &start_iter, &end_iter,
+                                  FALSE);
+
+  XST_mPV(0, text);
+    g_free(text);
+  if (GIMME_V == G_ARRAY) {
+    XST_mIV(1, k_circular_queue_nth(world->gui->line_times,
+                                    time_t,
+                                    line));
+    XSRETURN(2);
+  } else {
+    XSRETURN(1);
+  }
+}
+
+
 static
 void
 XS_play(pTHX_ CV *perlcv)
@@ -1161,6 +1347,13 @@ XS_trigger(pTHX_ CV *perlcv)
         XSRETURN_EMPTY;
       }
       break;
+
+    default:
+      warn(_("%s: Wrong argument type"),  "trigger");
+      if (i == -1) {
+        g_free(trigger);
+      }
+      XSRETURN_EMPTY;
     }
     ++argpos;
   }
@@ -1685,6 +1878,13 @@ XS_alias(pTHX_ CV *perlcv)
         XSRETURN_EMPTY;
       }
       break;
+
+    default:
+      warn(_("%s: Wrong argument type"),  "alias");
+      if (i == -1) {
+        g_free(alias);
+      }
+      XSRETURN_EMPTY;
     }
     ++argpos;
   }
@@ -1742,6 +1942,59 @@ XS_alias(pTHX_ CV *perlcv)
 }
 
 
+static
+void
+XS_expandalias(pTHX_ CV *perlcv)
+{
+  GSList *aliasptr;
+  Alias  *alias;
+  char   *line;
+  char   *resultline;
+  SV     *re;
+  SV     *tmp;
+  FETCHWORLDVARS;
+  dXSARGS;
+
+  FETCHWORLD;
+
+  if (items == 1) {
+    warn(_("%s: Too few arguments"), "expandalias");
+    XSRETURN_EMPTY;
+  }
+
+  line = SvPV_nolen(ST(1));
+  tmp = get_sv("_tmpexpalias", TRUE);
+  sv_setpv(tmp, line);
+
+  aliasptr = world->aliases;
+  while (aliasptr) {
+    gchar *subcmd;
+
+    alias = (Alias *) aliasptr->data;
+
+    if (!alias->enabled) {
+      break;
+    }
+
+    /* Set the regexp in a variable */
+    re = get_sv("_re", TRUE);
+    sv_setsv(re, alias->pattern_re);
+    subcmd = g_strdup_printf("$_tmpexpalias =~ s\x05$_re\x05%s\x05%s;",
+                             alias->substitution,
+                             alias->perl_eval ? "e" : "");
+    eval_pv(subcmd, TRUE);
+    g_free(subcmd);
+
+    aliasptr = aliasptr->next;
+  }
+
+  resultline = SvPV_nolen(tmp);
+  XST_mPV(0, resultline);
+
+  XSRETURN(1);
+}
+
+
 static
 void
 XS__listalias(pTHX_ CV *perlcv)
@@ -1999,6 +2252,13 @@ XS_macro(pTHX_ CV *perlcv)
         XSRETURN_EMPTY;
       }
       break;
+
+    default:
+      warn(_("%s: Wrong argument type"),  "macro");
+      if (i == -1) {
+        g_free(macro);
+      }
+      XSRETURN_EMPTY;
     }
     ++argpos;
   }
@@ -2444,7 +2704,7 @@ XS_makepermanent(pTHX_ CV *perlcv)
   for (i = 1; i < items; ++i) {
     var = SvPV_nolen(ST(i));
     if (var[0] != '$' && var[0] != '@' && var[0] != '%') {
-      warn(_("%s: Only scalar ($), array (@) and hash (%) variables can be made permanent"),
+      warn(_("%s: Only scalar ($), array (@) and hash (%%) variables can be made permanent"),
            "makepermanent");
       continue;
     }
@@ -2662,6 +2922,12 @@ XS_hook(pTHX_ CV *perlcv)
         XSRETURN_EMPTY;
       }
       break;
+
+    default:
+      warn(_("%s: Wrong argument type"),  "hook");
+      g_free(action);
+      g_free(name);
+      XSRETURN_EMPTY;
     }
     ++argpos;
   }
@@ -2797,7 +3063,7 @@ XS_logfile(pTHX_ CV *perlcv)
     g_free(world->log_file_name);
     world->log_file_name = g_strdup(tmp);
 
-    if (start_log(world, &errmsg)) {
+    if (start_log(world, 0, &errmsg)) {
       ansitextview_append_stringf(world->gui,
                                   _("Logging to file %s.\n"),
                                   world->log_actual_file);
@@ -2851,7 +3117,7 @@ XS__writetolog(pTHX_ CV *perlcv)
 
   for (i = 1; i < items; ++i) {
     line = SvPV_nolen(ST(i));
-    write_log_line(world, line);
+    write_log_line(world, line, NULL);
   }
 
   XSRETURN_EMPTY;
@@ -3160,9 +3426,11 @@ xs_init()
   newXS("World::getconntime",  XS_getconntime,  "World");
   newXS("World::getidletime",  XS_getidletime,  "World");
 
-  newXS("World::echo",      XS_echo,     "World");
-  newXS("World::send",      XS_send,     "World");
-  newXS("World::sendfile",  XS_sendfile, "World");
+  newXS("World::echo",      XS_echo,      "World");
+  newXS("World::send",      XS_send,      "World");
+  newXS("World::_mlsend",   XS__mlsend,   "World");
+  newXS("World::interpret", XS_interpret, "World");
+  newXS("World::getline",   XS_getline,   "World");
 
   newXS("play", XS_play, "");
 
@@ -3177,10 +3445,11 @@ xs_init()
   newXS("World::_getobjectnumber", XS__getobjectnumber, "World");
   newXS("World::_objectenabled",   XS__objectenabled,   "World");
 
-  newXS("World::alias",      XS_alias,      "World");
-  newXS("World::_listalias", XS__listalias, "World");
-  newXS("World::_delalias",  XS__delalias,  "World");
-  newXS("World::_movealias", XS__movealias, "World");
+  newXS("World::alias",       XS_alias,       "World");
+  newXS("World::expandalias", XS_expandalias, "World");
+  newXS("World::_listalias",  XS__listalias,  "World");
+  newXS("World::_delalias",   XS__delalias,   "World");
+  newXS("World::_movealias",  XS__movealias,  "World");
 
   newXS("World::getkeycode", XS_getkeycode, "World");
   newXS("World::macro",      XS_macro,      "World");
