@@ -1,7 +1,7 @@
-/* $Id: prefs.c,v 1.43 2008/03/23 21:09:33 ekalin Exp $ */
+/* $Id: prefs.c 1014 2009-02-01 12:46:52Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2008 Eduardo M Kalinowski <ekalin@gmail.com>
+ * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -67,7 +67,7 @@ static char *colorNames[] = {
 /***********************
  * Function prototypes *
  ***********************/
-static void     write_global_prefs_to(FILE *fp);
+static GString *get_preferences_string(void);
 static void     read_global_preferences_from(FILE *fp,
                                              int *x, int *y,
                                              int *width, int *height);
@@ -75,6 +75,7 @@ static void     combo_changed_cb(GtkComboBox *combo, gpointer data);
 static gboolean txt_cmd_focus_out_cb(GtkWidget     *widget,
                                      GdkEventFocus *event,
                                      gpointer       data);
+static void     cmbProxyType_changed_cb(GtkComboBox *combo, gpointer data);
 /* Glade signals */
 G_MODULE_EXPORT void     cmd_group_size_changed_cb(GtkSpinButton *spin,
                                                    gpointer       data);
@@ -82,6 +83,8 @@ G_MODULE_EXPORT void     cmd_delay_changed_cb(GtkSpinButton *spin,
                                                    gpointer       data);
 G_MODULE_EXPORT void     tab_position_changed_cb(GtkComboBox *combo,
                                                  gpointer data);
+G_MODULE_EXPORT void     chk_hide_single_tab_toggled_cb(GtkToggleButton *button,
+                                                        gpointer data);
 G_MODULE_EXPORT void     chk_urgency_toggled_cb(GtkToggleButton *button,
                                                 gpointer data);
 
@@ -91,18 +94,24 @@ show_preferences_dialog_cb(GtkWidget *widget, gpointer data)
 {
   static GtkWidget *dialog = NULL;
   GladeXML         *xml;
+  GtkNotebook      *notebook;
   GtkWidget        *cmbInfo;
   GtkWidget        *cmbCmdEcho;
   GtkWidget        *cmbTabPos;
+  GtkWidget        *chkHideSingleTab;
   GtkWidget        *chkUrgency;
   GtkWidget        *spnGroupSize;
   GtkWidget        *spnDelay;
+  GtkComboBox      *cmbProxyType;
+  GtkWidget        *txtProxyServer;
+  GtkWidget        *txtProxyPort;
+  GtkWidget        *txtProxyUser;
+  GtkWidget        *txtProxyPassword;
 #ifndef __WIN32__
   GtkWidget        *txtBrowserCommand;
   GtkWidget        *txtAudioPlayerCommand;
 #else
   GtkWidget        *tabPrograms;
-  GtkNotebook      *notebook;
 #endif
   GtkBox           *vbox;
 
@@ -110,6 +119,7 @@ show_preferences_dialog_cb(GtkWidget *widget, gpointer data)
     xml = glade_xml_new(get_kildclient_installed_file("kildclient.glade"),
                         "dlgPreferences", NULL);
     dialog = glade_xml_get_widget(xml, "dlgPreferences");
+    notebook = (GtkNotebook *) glade_xml_get_widget(xml, "ntbkPrefs");
     vbox = GTK_BOX(glade_xml_get_widget(xml, "vbox_int_kc_colors"));
     glade_xml_signal_autoconnect(xml);
 
@@ -140,6 +150,10 @@ show_preferences_dialog_cb(GtkWidget *widget, gpointer data)
     gtk_combo_box_set_active(GTK_COMBO_BOX(cmbTabPos),
                              globalPrefs.tab_position);
 
+    chkHideSingleTab = glade_xml_get_widget(xml, "chkHideSingleTab");
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chkHideSingleTab),
+                                 globalPrefs.hide_single_tab);
+
     /* Load use of urgency hint */
     chkUrgency = glade_xml_get_widget(xml, "chkUrgency");
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chkUrgency),
@@ -149,34 +163,64 @@ show_preferences_dialog_cb(GtkWidget *widget, gpointer data)
     spnGroupSize = glade_xml_get_widget(xml, "spnGroupSize");
     gtk_spin_button_set_value(GTK_SPIN_BUTTON(spnGroupSize),
                               globalPrefs.multi_cmd_group_size);
-    spnDelay = glade_xml_get_widget(xml, "spnDelay");
+    spnDelay = glade_xml_get_widget(xml, "spnPrefsDelay");
     gtk_spin_button_set_value(GTK_SPIN_BUTTON(spnDelay),
                               globalPrefs.multi_cmd_delay);
 
+    /* Load proxy settings */
+    cmbProxyType = GTK_COMBO_BOX(glade_xml_get_widget(xml, "cmbGProxyType"));
+    txtProxyServer   = glade_xml_get_widget(xml, "txtGProxyServer");
+    gtk_entry_set_text(GTK_ENTRY(txtProxyServer), globalPrefs.proxy.server);
+    g_signal_connect(G_OBJECT(txtProxyServer), "focus_out_event",
+                     G_CALLBACK(txt_cmd_focus_out_cb),
+                     &globalPrefs.proxy.server);
+    txtProxyPort     = glade_xml_get_widget(xml, "txtGProxyPort");
+    gtk_entry_set_text(GTK_ENTRY(txtProxyPort), globalPrefs.proxy.port);
+    g_signal_connect(G_OBJECT(txtProxyPort), "focus_out_event",
+                     G_CALLBACK(txt_cmd_focus_out_cb),
+                     &globalPrefs.proxy.port);
+    txtProxyUser     = glade_xml_get_widget(xml, "txtGProxyUser");
+    gtk_entry_set_text(GTK_ENTRY(txtProxyUser), globalPrefs.proxy.user);
+    g_signal_connect(G_OBJECT(txtProxyUser), "focus_out_event",
+                     G_CALLBACK(txt_cmd_focus_out_cb),
+                     &globalPrefs.proxy.user);
+    txtProxyPassword = glade_xml_get_widget(xml, "txtGProxyPassword");
+    gtk_entry_set_text(GTK_ENTRY(txtProxyPassword), globalPrefs.proxy.password);
+    g_signal_connect(G_OBJECT(txtProxyPassword), "focus_out_event",
+                     G_CALLBACK(txt_cmd_focus_out_cb),
+                     &globalPrefs.proxy.password);
+    /* Remove "Use Global Settings" - these are the global settings */
+    gtk_combo_box_remove_text(cmbProxyType, 0);
+    gtk_combo_box_set_active(cmbProxyType, globalPrefs.proxy.type);
+    g_signal_connect(G_OBJECT(cmbProxyType), "changed",
+                     G_CALLBACK(cmbProxyType_changed_cb), NULL);
+
 #ifndef __WIN32__
     /* Load commands */
     txtBrowserCommand = glade_xml_get_widget(xml, "txtBrowserCommand");
+    gtk_entry_set_text(GTK_ENTRY(txtBrowserCommand),
+                       globalPrefs.browser_command);
     g_signal_connect(G_OBJECT(txtBrowserCommand), "focus_out_event",
                      G_CALLBACK(txt_cmd_focus_out_cb),
                      &globalPrefs.browser_command);
-    gtk_entry_set_text(GTK_ENTRY(txtBrowserCommand),
-                       globalPrefs.browser_command);
     txtAudioPlayerCommand = glade_xml_get_widget(xml, "txtAudioPlayerCommand");
+    gtk_entry_set_text(GTK_ENTRY(txtAudioPlayerCommand),
+                       globalPrefs.audio_player_command);
     g_signal_connect(G_OBJECT(txtAudioPlayerCommand), "focus_out_event",
                      G_CALLBACK(txt_cmd_focus_out_cb),
                      &globalPrefs.audio_player_command);
-    gtk_entry_set_text(GTK_ENTRY(txtAudioPlayerCommand),
-                       globalPrefs.audio_player_command);
 #else /* defined __WIN32__ */
-    notebook = (GtkNotebook *) glade_xml_get_widget(xml, "ntbkPrefs");
     tabPrograms = glade_xml_get_widget(xml, "prefs_panel_progs");
     gtk_notebook_remove_page(notebook,
                              gtk_notebook_page_num(notebook, tabPrograms));
 #endif
     g_object_unref(xml);
+
+    /* For some reason the proxy page insists on being the default selected. */
+    gtk_widget_show_all(dialog);
+    gtk_notebook_set_current_page(notebook, 0);
   }
 
-  gtk_widget_show_all(dialog);
   gtk_window_present(GTK_WINDOW(dialog));
 }
 
@@ -213,6 +257,19 @@ tab_position_changed_cb(GtkComboBox *combo, gpointer data)
 }
 
 
+void
+chk_hide_single_tab_toggled_cb(GtkToggleButton *button, gpointer data)
+{
+  globalPrefs.hide_single_tab = gtk_toggle_button_get_active(button);
+  if (globalPrefs.hide_single_tab) {
+    gtk_notebook_set_show_tabs(GTK_NOTEBOOK(ntbkWorlds),
+                               gtk_notebook_get_n_pages(GTK_NOTEBOOK(ntbkWorlds)) != 1);
+  } else {
+    gtk_notebook_set_show_tabs(GTK_NOTEBOOK(ntbkWorlds), TRUE);
+  }
+}
+
+
 void
 chk_urgency_toggled_cb(GtkToggleButton *button, gpointer data)
 {
@@ -239,6 +296,14 @@ txt_cmd_focus_out_cb(GtkWidget     *widget,
 }
 
 
+static
+void
+cmbProxyType_changed_cb(GtkComboBox *combo, gpointer data)
+{
+  globalPrefs.proxy.type = gtk_combo_box_get_active(combo);
+}
+
+
 void
 read_global_preferences(int *x, int *y, int *width, int *height)
 {
@@ -254,8 +319,11 @@ read_global_preferences(int *x, int *y, int *width, int *height)
   globalPrefs.multi_cmd_group_size = 1;
   globalPrefs.multi_cmd_delay = 0;
 
-  globalPrefs.tab_position = GTK_POS_TOP;
-  globalPrefs.urgency_hint = TRUE;
+  globalPrefs.tab_position    = GTK_POS_TOP;
+  globalPrefs.hide_single_tab = FALSE;
+  globalPrefs.urgency_hint    = TRUE;
+
+  globalPrefs.proxy.type = PROXY_NONE;
 
   /* And now try to load from file the actual values */
   kilddir = get_kildclient_directory_path();
@@ -275,6 +343,20 @@ read_global_preferences(int *x, int *y, int *width, int *height)
     globalPrefs.audio_player_command = g_strdup("play \"%s\" &");
   }
 
+  /* Have the proxy settings been set? */
+  if (!globalPrefs.proxy.server) {
+    globalPrefs.proxy.server   = g_strdup("");
+  }
+  if (!globalPrefs.proxy.port) {
+    globalPrefs.proxy.port     = g_strdup("1080");
+  }
+  if (!globalPrefs.proxy.user) {
+    globalPrefs.proxy.user     = g_strdup("");
+  }
+  if (!globalPrefs.proxy.password) {
+    globalPrefs.proxy.password = g_strdup("");
+  }
+
   g_free(filepath);
 }
 
@@ -311,6 +393,8 @@ read_global_preferences_from(FILE *fp,
       }
     } else if (strcmp(first_word, "tabposition") == 0) {
       globalPrefs.tab_position = atoi(line + pos + 1);
+    } else if (strcmp(first_word, "hidesingletab") == 0) {
+      globalPrefs.hide_single_tab = atoi(line + pos + 1);
     } else if (strcmp(first_word, "urgencyhint") == 0) {
       globalPrefs.urgency_hint = atoi(line + pos + 1);
     } else if (strcmp(first_word, "browsercommand") == 0) {
@@ -335,6 +419,17 @@ read_global_preferences_from(FILE *fp,
     } else if (strcmp(first_word, "nopluginhelpmsg") == 0) {
       globalPrefs.no_plugin_help_msg = atoi(line + pos + 1);
 
+    } else if (strcmp(first_word, "proxytype") == 0) {
+      globalPrefs.proxy.type = atoi(line + pos + 1);
+    } else if (strcmp(first_word, "proxyserver") == 0) {
+      globalPrefs.proxy.server = g_strdup(line + pos + 1);
+    } else if (strcmp(first_word, "proxyport") == 0) {
+      globalPrefs.proxy.port = g_strdup(line + pos + 1);
+    } else if (strcmp(first_word, "proxyuser") == 0) {
+      globalPrefs.proxy.user = g_strdup(line + pos + 1);
+    } else if (strcmp(first_word, "proxypassword") == 0) {
+      globalPrefs.proxy.password = g_strdup(line + pos + 1);
+
     } else {
       fprintf(stderr,
               _("Unknown directive '%s' in config file.\n"), first_word);
@@ -350,58 +445,82 @@ save_global_preferences(void)
 {
   const char *dirpath;
   char       *filepath;
-  FILE       *fp;
+  GString    *file_contents;
+  GError     *error = NULL;
+
+  file_contents = get_preferences_string();
 
   dirpath = create_kildclient_directory();
-
   filepath = g_strdup_printf("%s/kildclient.cfg", dirpath);
-  fp = fopen(filepath, "w");
-  if (!fp) {
+  if (!g_file_set_contents(filepath,
+                           file_contents->str, file_contents->len,
+                           &error)) {
     GtkWidget *dlg = gtk_message_dialog_new(GTK_WINDOW(wndMain),
                                             GTK_DIALOG_DESTROY_WITH_PARENT,
                                             GTK_MESSAGE_ERROR,
                                             GTK_BUTTONS_CLOSE,
                                             _("Could not open file '%s': %s"),
                                             filepath,
-                                            g_strerror(errno));
+                                            error->message);
     gtk_window_set_title(GTK_WINDOW(dlg), _("KildClient Error"));
     gtk_dialog_run(GTK_DIALOG(dlg));
     gtk_widget_destroy(dlg);
-    g_free(filepath);
-    return;
   }
 
-  write_global_prefs_to(fp);
-  fclose(fp);
   g_free(filepath);
+  g_string_free(file_contents, TRUE);
 }
 
 
 static
-void
-write_global_prefs_to(FILE *fp)
+GString *
+get_preferences_string(void)
 {
+  GString *str;
   int x, y, width, height;
 
-  fprintf(fp, "newinfomsgcolor %d\n", globalPrefs.idxInfoMsgColor);
-  fprintf(fp, "newcmdechocolor %d\n", globalPrefs.idxCmdEchoColor);
+  str = g_string_sized_new(512);
 
-  fprintf(fp, "tabposition %d\n", globalPrefs.tab_position);
-  fprintf(fp, "urgencyhint %d\n", globalPrefs.urgency_hint);
+  g_string_append_printf(str,
+                         "newinfomsgcolor %d\n", globalPrefs.idxInfoMsgColor);
+  g_string_append_printf(str,
+                         "newcmdechocolor %d\n", globalPrefs.idxCmdEchoColor);
 
-  fprintf(fp, "browsercommand %s\n", globalPrefs.browser_command);
-  fprintf(fp, "audioplayercommand %s\n", globalPrefs.audio_player_command);
+  g_string_append_printf(str, "tabposition %d\n", globalPrefs.tab_position);
+  g_string_append_printf(str, "hidesingletab %d\n",
+                         globalPrefs.hide_single_tab);
+  g_string_append_printf(str, "urgencyhint %d\n", globalPrefs.urgency_hint);
 
-  fprintf(fp, "lastopenworld %s\n", globalPrefs.last_open_world);
+  g_string_append_printf(str,
+                         "browsercommand %s\n", globalPrefs.browser_command);
+  g_string_append_printf(str,
+                         "audioplayercommand %s\n",
+                         globalPrefs.audio_player_command);
 
-  fprintf(fp, "multicmdgroupsize %d\n", globalPrefs.multi_cmd_group_size);
-  fprintf(fp, "multicmddelay %f\n",     globalPrefs.multi_cmd_delay);
+  g_string_append_printf(str,
+                         "lastopenworld %s\n", globalPrefs.last_open_world);
 
-  fprintf(fp, "nopluginhelpmsg %d\n", globalPrefs.no_plugin_help_msg);
+  g_string_append_printf(str,
+                         "multicmdgroupsize %d\n",
+                         globalPrefs.multi_cmd_group_size);
+  g_string_append_printf(str,
+                         "multicmddelay %f\n", globalPrefs.multi_cmd_delay);
+
+  g_string_append_printf(str,
+                         "nopluginhelpmsg %d\n",
+                         globalPrefs.no_plugin_help_msg);
 
   gtk_window_get_position(GTK_WINDOW(wndMain), &x, &y);
   gtk_window_get_size(GTK_WINDOW(wndMain), &width, &height);
-  fprintf(fp, "window %d %d %d %d\n", x, y, width, height);
+  g_string_append_printf(str, "window %d %d %d %d\n", x, y, width, height);
+
+  g_string_append_printf(str, "proxytype %d\n",     globalPrefs.proxy.type);
+  g_string_append_printf(str, "proxyserver %s\n",   globalPrefs.proxy.server);
+  g_string_append_printf(str, "proxyport %s\n",     globalPrefs.proxy.port);
+  g_string_append_printf(str, "proxyuser %s\n",     globalPrefs.proxy.user);
+  g_string_append_printf(str, "proxypassword %s\n", globalPrefs.proxy.password);
+
+  return str;
 }
 
 
