@@ -1877,7 +1877,9 @@ simo_combo_box_txtview_keypress_cb(GtkWidget   *widget,
                                    gpointer    data)
 {
   if ((evt->keyval == GDK_Return || evt->keyval == GDK_KP_Enter)
-      && !(evt->state & GDK_MOD1_MASK)) {
+      && !(evt->state & GDK_MOD1_MASK)
+      && !(evt->state & GDK_SHIFT_MASK)
+      && !(evt->state & GDK_CONTROL_MASK)) {
     g_signal_emit(SIMO_COMBO_BOX(data), combo_box_signals[ACTIVATE], 0);
     return TRUE;
   }
