@@ -1,7 +1,7 @@
-/* $Id: timers.c,v 1.17 2008/03/23 21:09:31 ekalin Exp $ */
+/* $Id: timers.c 1014 2009-02-01 12:46:52Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2008 Eduardo M Kalinowski <ekalin@gmail.com>
+ * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -240,17 +240,17 @@ list_timers(World *world, Plugin *plugin)
 
 
 void
-save_timer(FILE *fp, Timer *timer)
+save_timer(GString *str, Timer *timer)
 {
-  fprintf(fp, "    <timer ");
+  g_string_append(str, "    <timer ");
   if (timer->name) {
-    fprintf(fp, "name=\"%s\" ", timer->name);
+    g_string_append_printf(str, "name=\"%s\" ", timer->name);
   }
-  fprintf(fp,
-          "interval=\"%d\" count=\"%d\" enabled=\"%d\" temporary=\"%d\">",
-          timer->interval,
-          timer->count,
-          timer->enabled,
-          timer->temporary);
-  fprintf_escaped(fp, "%s</timer>\n", timer->action);
+  g_string_append_printf(str,
+                         "interval=\"%d\" count=\"%d\" enabled=\"%d\" temporary=\"%d\">",
+                         timer->interval,
+                         timer->count,
+                         timer->enabled,
+                         timer->temporary);
+  kc_g_string_append_escaped(str, "%s</timer>\n", timer->action);
 }
