@@ -1,7 +1,7 @@
-/* $Id: triggers.c,v 1.39 2008/03/23 21:09:31 ekalin Exp $ */
+/* $Id: triggers.c 1024 2009-02-09 18:01:06Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2008 Eduardo M Kalinowski <ekalin@gmail.com>
+ * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -99,7 +99,7 @@ match_triggers(World     *world,
       ansitextview_append_ansi_string(world->gui,
                                       (gchar *) *colorline, *output_len);
       if (world->log_file) {
-        write_log_line(world, (gchar *) *line);
+        write_log_line(world, (gchar *) *line, NULL);
       }
     }
 
@@ -208,7 +208,7 @@ match_triggers(World     *world,
       world->gui->ta.saved_state = world->gui->ta.state;
     }
     if (world->log_file && !*gag_log) {
-      write_log_line(world, (gchar *) *line);
+      write_log_line(world, (gchar *) *line, NULL);
     }
   }
 
@@ -484,30 +484,31 @@ trigger_precompute_res(World *world)
 
 
 void
-save_trigger(FILE *fp, Trigger *trigger)
+save_trigger(GString *str, Trigger *trigger)
 {
-  fprintf(fp, "    <trigger ");
+  g_string_append(str, "    <trigger ");
   if (trigger->name) {
-    fprintf_escaped(fp, "name=\"%s\" ", trigger->name);
+    kc_g_string_append_escaped(str, "name=\"%s\" ", trigger->name);
   }
-  fprintf(fp, "enabled=\"%d\" keepexecuting=\"%d\" rewriter=\"%d\" ignorecase=\"%d\"%s%s>\n",
-          trigger->enabled,
-          trigger->keepexecuting,
-          trigger->rewriter,
-          trigger->ignore_case,
-          trigger->gag_output ? " gag=\"1\"" : "",
-          trigger->gag_log    ? " gaglog=\"1\"": "");
-  fprintf_escaped(fp, "      <pattern>%s</pattern>\n",
-                  trigger->pattern);
+  g_string_append_printf(str,
+                         "enabled=\"%d\" keepexecuting=\"%d\" rewriter=\"%d\" ignorecase=\"%d\"%s%s>\n",
+                         trigger->enabled,
+                         trigger->keepexecuting,
+                         trigger->rewriter,
+                         trigger->ignore_case,
+                         trigger->gag_output ? " gag=\"1\"" : "",
+                         trigger->gag_log    ? " gaglog=\"1\"": "");
+  kc_g_string_append_escaped(str,
+                             "      <pattern>%s</pattern>\n", trigger->pattern);
   if (trigger->action) {
-    fprintf_escaped(fp, "      <action>%s</action>\n",
-                    trigger->action);
+    kc_g_string_append_escaped(str,
+                               "      <action>%s</action>\n", trigger->action);
   }
-  fprintf(fp, "      <highlight enabled=\"%d\" target=\"%d\" fg=\"%d\" bg=\"%d\" italics=\"%d\" strike=\"%d\" underline=\"%d\"/>\n",
-          trigger->highlight, trigger->high_target,
-          trigger->high_fg_color, trigger->high_bg_color,
-          trigger->high_italic, trigger->high_strike, trigger->high_underline);
-  fprintf(fp, "    </trigger>\n");
+  g_string_append_printf(str, "      <highlight enabled=\"%d\" target=\"%d\" fg=\"%d\" bg=\"%d\" italics=\"%d\" strike=\"%d\" underline=\"%d\"/>\n",
+                         trigger->highlight, trigger->high_target,
+                         trigger->high_fg_color, trigger->high_bg_color,
+                         trigger->high_italic, trigger->high_strike, trigger->high_underline);
+  g_string_append(str, "    </trigger>\n");
 }
 
 
@@ -552,11 +553,11 @@ create_test_triggers_dialog(GtkWindow *parent, World *world)
 
   /* Signals */
   glade_xml_signal_autoconnect(gladexml);
-  btnClose = glade_xml_get_widget(gladexml, "btnClose");
+  btnClose = glade_xml_get_widget(gladexml, "btnTTClose");
   g_signal_connect_swapped(G_OBJECT(btnClose), "clicked",
                            G_CALLBACK(gtk_widget_hide), dlg);
 
-  btnSend = glade_xml_get_widget(gladexml, "btnSend");
+  btnSend = glade_xml_get_widget(gladexml, "btnTriggerSend");
   g_signal_connect(G_OBJECT(btnSend), "clicked",
                    G_CALLBACK(test_trigger_send_cb), world);
 
