@@ -1,7 +1,7 @@
-/* $Id: we_generic_guied.c,v 1.11 2008/03/23 21:09:28 ekalin Exp $ */
+/* $Id: we_generic_guied.c 1014 2009-02-01 12:46:52Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2008 Eduardo M Kalinowski <ekalin@gmail.com>
+ * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -45,7 +45,7 @@ typedef void (*UpdateFunc)();
 static void          we_guied_do_export(World       *world,
                                         const gchar *file,
                                         GladeXML    *gladexml);
-static void          we_guied_export_selected_hooks(FILE         *fp,
+static void          we_guied_export_selected_hooks(GString      *str,
                                                     GSList       *list,
                                                     const gchar  *name,
                                                     GtkTreeModel *model,
@@ -107,30 +107,18 @@ static
 void
 we_guied_do_export(World *world, const gchar *file, GladeXML *gladexml)
 {
-  FILE             *fp;
   GtkTreeSelection *selection;
   GtkTreeModel     *model;
   GList            *selected;
   GList            *listiter;
   GtkTreeIter       iter;
+  GString          *file_contents;
+  GError           *error = NULL;
 
-  fp = fopen(file, "w");
-  if (!fp) {
-    GtkWidget *dlg = gtk_message_dialog_new(GTK_WINDOW(world->dlgEditWorld),
-                                            GTK_DIALOG_DESTROY_WITH_PARENT,
-                                            GTK_MESSAGE_ERROR,
-                                            GTK_BUTTONS_CLOSE,
-                                            _("Could not open file '%s': %s"),
-                                            file,
-                                            g_strerror(errno));
-    gtk_window_set_title(GTK_WINDOW(dlg), _("KildClient Error"));
-    gtk_dialog_run(GTK_DIALOG(dlg));
-    gtk_widget_destroy(dlg);
-    return;
-  }
+  file_contents = g_string_sized_new(1024);
 
   /* Header */
-  fprintf(fp, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
+  g_string_append(file_contents, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
               "<!DOCTYPE kcworld-export SYSTEM \"kcworld.dtd\">\n\n"
               "<kcworld-export>\n");
 
@@ -139,7 +127,7 @@ we_guied_do_export(World *world, const gchar *file, GladeXML *gladexml)
     selection = gtk_tree_view_get_selection(world->viewTrigger);
     selected  = gtk_tree_selection_get_selected_rows(selection, &model);
     if (selected) {
-      fprintf(fp, "  <triggers>\n");
+      g_string_append(file_contents, "  <triggers>\n");
 
       listiter = selected;
       while (listiter) {
@@ -147,14 +135,14 @@ we_guied_do_export(World *world, const gchar *file, GladeXML *gladexml)
 
         gtk_tree_model_get_iter(model, &iter, listiter->data);
         gtk_tree_model_get(model, &iter, WE_GUIED_POINTER, &trigger, -1);
-        save_trigger(fp, trigger);
+        save_trigger(file_contents, trigger);
 
         listiter = listiter->next;
       }
       g_list_foreach(selected, (GFunc) gtk_tree_path_free, NULL);
       g_list_free(selected);
 
-      fprintf(fp, "  </triggers>\n");
+      g_string_append(file_contents, "  </triggers>\n");
     }
   }
 
@@ -163,7 +151,7 @@ we_guied_do_export(World *world, const gchar *file, GladeXML *gladexml)
     selection = gtk_tree_view_get_selection(world->viewAlias);
     selected  = gtk_tree_selection_get_selected_rows(selection, &model);
     if (selected) {
-      fprintf(fp, "  <aliases>\n");
+      g_string_append(file_contents, "  <aliases>\n");
 
       listiter = selected;
       while (listiter) {
@@ -171,14 +159,14 @@ we_guied_do_export(World *world, const gchar *file, GladeXML *gladexml)
 
         gtk_tree_model_get_iter(model, &iter, listiter->data);
         gtk_tree_model_get(model, &iter, WE_GUIED_POINTER, &alias, -1);
-        save_alias(fp, alias);
+        save_alias(file_contents, alias);
 
         listiter = listiter->next;
       }
       g_list_foreach(selected, (GFunc) gtk_tree_path_free, NULL);
       g_list_free(selected);
 
-      fprintf(fp, "  </aliases>\n");
+      g_string_append(file_contents, "  </aliases>\n");
     }
   }
 
@@ -187,7 +175,7 @@ we_guied_do_export(World *world, const gchar *file, GladeXML *gladexml)
     selection = gtk_tree_view_get_selection(world->viewMacro);
     selected  = gtk_tree_selection_get_selected_rows(selection, &model);
     if (selected) {
-      fprintf(fp, "  <macros>\n");
+      g_string_append(file_contents, "  <macros>\n");
 
       listiter = selected;
       while (listiter) {
@@ -195,14 +183,14 @@ we_guied_do_export(World *world, const gchar *file, GladeXML *gladexml)
 
         gtk_tree_model_get_iter(model, &iter, listiter->data);
         gtk_tree_model_get(model, &iter, WE_GUIED_POINTER, &macro, -1);
-        save_macro(fp, macro);
+        save_macro(file_contents, macro);
 
         listiter = listiter->next;
       }
       g_list_foreach(selected, (GFunc) gtk_tree_path_free, NULL);
       g_list_free(selected);
 
-      fprintf(fp, "  </macros>\n");
+      g_string_append(file_contents, "  </macros>\n");
     }
   }
 
@@ -211,7 +199,7 @@ we_guied_do_export(World *world, const gchar *file, GladeXML *gladexml)
     selection = gtk_tree_view_get_selection(world->viewTimer);
     selected  = gtk_tree_selection_get_selected_rows(selection, &model);
     if (selected) {
-      fprintf(fp, "  <timers>\n");
+      g_string_append(file_contents, "  <timers>\n");
 
       listiter = selected;
       while (listiter) {
@@ -219,14 +207,14 @@ we_guied_do_export(World *world, const gchar *file, GladeXML *gladexml)
 
         gtk_tree_model_get_iter(model, &iter, listiter->data);
         gtk_tree_model_get(model, &iter, WE_GUIED_POINTER, &timer, -1);
-        save_timer(fp, timer);
+        save_timer(file_contents, timer);
 
         listiter = listiter->next;
       }
       g_list_foreach(selected, (GFunc) gtk_tree_path_free, NULL);
       g_list_free(selected);
 
-      fprintf(fp, "  </timers>\n");
+      g_string_append(file_contents, "  </timers>\n");
     }
   }
 
@@ -235,7 +223,7 @@ we_guied_do_export(World *world, const gchar *file, GladeXML *gladexml)
     selection = gtk_tree_view_get_selection(world->viewVar);
     selected  = gtk_tree_selection_get_selected_rows(selection, &model);
     if (selected) {
-      fprintf(fp, "  <variables>\n");
+      g_string_append(file_contents, "  <variables>\n");
 
       listiter = selected;
       while (listiter) {
@@ -243,64 +231,82 @@ we_guied_do_export(World *world, const gchar *file, GladeXML *gladexml)
 
         gtk_tree_model_get_iter(model, &iter, listiter->data);
         gtk_tree_model_get(model, &iter, WE_GUIED_POINTER, &name, -1);
-        fprintf(fp, "    <variable>%s</variable>\n", name);
+        g_string_append_printf(file_contents,
+                               "    <variable>%s</variable>\n",
+                               name);
 
         listiter = listiter->next;
       }
       g_list_foreach(selected, (GFunc) gtk_tree_path_free, NULL);
       g_list_free(selected);
 
-      fprintf(fp, "  </variables>\n");
+      g_string_append(file_contents, "  </variables>\n");
     }
   }
 
   /* Hooks */
-  fprintf(fp, "  <hooksv2>\n");
-  we_guied_export_selected_hooks(fp,
+  g_string_append(file_contents, "  <hooksv2>\n");
+  we_guied_export_selected_hooks(file_contents,
                                  world->selected_OnConnect_hooks,
                                  "OnConnect",
                                  GTK_TREE_MODEL(world->hooks.OnConnect_model_filter),
                                  gladexml);
-  we_guied_export_selected_hooks(fp,
+  we_guied_export_selected_hooks(file_contents,
                                  world->selected_OnDisconnect_hooks,
                                  "OnDisconnect",
                                  GTK_TREE_MODEL(world->hooks.OnDisconnect_model_filter),
                                  gladexml);
-  we_guied_export_selected_hooks(fp,
+  we_guied_export_selected_hooks(file_contents,
                                  world->selected_OnReceivedText_hooks,
                                  "OnReceivedText",
                                  GTK_TREE_MODEL(world->hooks.OnReceivedText_model_filter),
                                  gladexml);
-  we_guied_export_selected_hooks(fp,
+  we_guied_export_selected_hooks(file_contents,
                                  world->selected_OnSentCommand_hooks,
                                  "OnSentCommand",
                                  GTK_TREE_MODEL(world->hooks.OnSentCommand_model_filter),
                                  gladexml);
-  we_guied_export_selected_hooks(fp,
+  we_guied_export_selected_hooks(file_contents,
                                  world->selected_OnGetFocus_hooks,
                                  "OnGetFocus",
                                  GTK_TREE_MODEL(world->hooks.OnGetFocus_model_filter),
                                  gladexml);
-  we_guied_export_selected_hooks(fp,
+  we_guied_export_selected_hooks(file_contents,
                                  world->selected_OnLoseFocus_hooks,
                                  "OnLoseFocus",
                                  GTK_TREE_MODEL(world->hooks.OnLoseFocus_model_filter),
                                  gladexml);
-  we_guied_export_selected_hooks(fp,
+  we_guied_export_selected_hooks(file_contents,
                                  world->selected_OnCloseConnected_hooks,
                                  "OnCloseConnected",
                                  GTK_TREE_MODEL(world->hooks.OnCloseConnected_model_filter),
                                  gladexml);
-  fprintf(fp, "  </hooksv2>\n");
+  g_string_append(file_contents, "  </hooksv2>\n");
 
-  fprintf(fp, "</kcworld-export>\n");
-  fclose(fp);
+  g_string_append(file_contents, "</kcworld-export>\n");
+
+
+  if (!g_file_set_contents(file,
+                           file_contents->str, file_contents->len,
+                           &error)) {
+    GtkWidget *dlg = gtk_message_dialog_new(GTK_WINDOW(world->dlgEditWorld),
+                                            GTK_DIALOG_DESTROY_WITH_PARENT,
+                                            GTK_MESSAGE_ERROR,
+                                            GTK_BUTTONS_CLOSE,
+                                            _("Could not open file '%s': %s"),
+                                            file,
+                                            error->message);
+    gtk_window_set_title(GTK_WINDOW(dlg), _("KildClient Error"));
+    gtk_dialog_run(GTK_DIALOG(dlg));
+    gtk_widget_destroy(dlg);
+  }
+  g_string_free(file_contents, TRUE);
 }
 
 
 static
 void
-we_guied_export_selected_hooks(FILE         *fp,
+we_guied_export_selected_hooks(GString      *str,
                                GSList       *list,
                                const gchar  *name,
                                GtkTreeModel *model,
@@ -317,7 +323,7 @@ we_guied_export_selected_hooks(FILE         *fp,
   if (list) {
     GtkTreePath *path;
 
-    fprintf(fp, "    <hooklistv2 for=\"%s\">\n", name);
+    g_string_append_printf(str, "    <hooklistv2 for=\"%s\">\n", name);
 
     listiter = list;
     while (listiter) {
@@ -325,13 +331,13 @@ we_guied_export_selected_hooks(FILE         *fp,
         path = gtk_tree_row_reference_get_path(listiter->data);
         gtk_tree_model_get_iter(model, &iter, path);
         gtk_tree_model_get(model, &iter, WE_GUIED_POINTER, &hook, -1);
-        save_hook(fp, hook);
+        save_hook(str, hook);
       }
       listiter = listiter->next;
     }
 
-    fprintf(fp, "    </hooklistv2>\n");
-    }
+    g_string_append(str, "    </hooklistv2>\n");
+  }
 }
 
 
