@@ -1,7 +1,7 @@
-/* $Id: we_input.c,v 1.15 2008/03/23 21:09:27 ekalin Exp $ */
+/* $Id: we_input.c 1014 2009-02-01 12:46:52Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2008 Eduardo M Kalinowski <ekalin@gmail.com>
+ * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -66,6 +66,7 @@ fill_we_input_tab(World *world, GladeXML *gladexml)
 {
   GtkWidget *chkRepeatCommands;
   GtkWidget *chkCmdEcho;
+  GtkWidget *chkNeverHideInput;
   GtkWidget *chkStoreCommands;
   GtkWidget *spnCommandsToSave;
   GtkWidget *txtCommandSeparator;
@@ -88,6 +89,7 @@ fill_we_input_tab(World *world, GladeXML *gladexml)
   /* Get widgets */
   chkRepeatCommands    = glade_xml_get_widget(gladexml, "chkRepeatCommands");
   chkCmdEcho           = glade_xml_get_widget(gladexml, "chkCmdEcho");
+  chkNeverHideInput    = glade_xml_get_widget(gladexml, "chkNeverHideInput");
   chkStoreCommands     = glade_xml_get_widget(gladexml, "chkStoreCommands");
   spnCommandsToSave    = glade_xml_get_widget(gladexml, "spnCommandsToSave");
   radSingleLine        = glade_xml_get_widget(gladexml, "radSingleLine");
@@ -108,6 +110,8 @@ fill_we_input_tab(World *world, GladeXML *gladexml)
                                world->repeat_commands);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chkCmdEcho),
                                world->cmd_echo);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chkNeverHideInput),
+                               world->never_hide_input);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chkStoreCommands),
 			       world->store_commands);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(spnCommandsToSave),
@@ -161,6 +165,7 @@ update_we_input_parameters(World *world)
   GladeXML  *gladexml;
   GtkWidget *chkRepeatCommands;
   GtkWidget *chkCmdEcho;
+  GtkWidget *chkNeverHideInput;
   GtkWidget *chkFloodPrevention;
   GtkWidget *spnMaxCommands;
   GtkWidget *txtPreventionCommand;
@@ -180,6 +185,7 @@ update_we_input_parameters(World *world)
   gladexml             = glade_get_widget_tree(world->dlgEditWorld);
   chkRepeatCommands    = glade_xml_get_widget(gladexml, "chkRepeatCommands");
   chkCmdEcho           = glade_xml_get_widget(gladexml, "chkCmdEcho");
+  chkNeverHideInput    = glade_xml_get_widget(gladexml, "chkNeverHideInput");
   chkStoreCommands     = glade_xml_get_widget(gladexml, "chkStoreCommands");
   spnCommandsToSave    = glade_xml_get_widget(gladexml, "spnCommandsToSave");
   chkAutoCompletion    = glade_xml_get_widget(gladexml, "chkAutoCompletion");
@@ -196,6 +202,11 @@ update_we_input_parameters(World *world)
     gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(chkRepeatCommands));
   world->cmd_echo =
     gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(chkCmdEcho));
+  world->never_hide_input =
+    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(chkNeverHideInput));
+  if (world->gui) {
+    simo_combo_box_set_visibility(world->gui->cmbEntry, TRUE);
+  }
 
   world->store_commands =
     gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(chkStoreCommands));
