@@ -1,7 +1,7 @@
-/* $Id: we_logging.c,v 1.3 2008/03/23 21:09:24 ekalin Exp $ */
+/* $Id: we_logging.c 1024 2009-02-09 18:01:06Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2008 Eduardo M Kalinowski <ekalin@gmail.com>
+ * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -56,16 +56,28 @@ get_we_logging_tab(GladeXML *gladexml, World *world)
 
   panel = glade_xml_get_widget(gladexml, "we_panel_logging");
 
-  btnStart = glade_xml_get_widget(gladexml, "btnLogStart");
-  image = gtk_image_new_from_stock(GTK_STOCK_MEDIA_PLAY,
-                                   GTK_ICON_SIZE_BUTTON);
-  gtk_button_set_image(GTK_BUTTON(btnStart), image);
-  g_signal_connect(G_OBJECT(btnStart), "clicked",
-                   G_CALLBACK(we_log_start_cb), world);
+  if (world != default_world) {
+    btnStart = glade_xml_get_widget(gladexml, "btnLogStart");
+    image = gtk_image_new_from_stock(GTK_STOCK_MEDIA_PLAY,
+                                     GTK_ICON_SIZE_BUTTON);
+    gtk_button_set_image(GTK_BUTTON(btnStart), image);
+    g_signal_connect(G_OBJECT(btnStart), "clicked",
+                     G_CALLBACK(we_log_start_cb), world);
 
-  btnStop = glade_xml_get_widget(gladexml, "btnLogStop");
-  g_signal_connect(G_OBJECT(btnStop), "clicked",
-                   G_CALLBACK(we_log_stop_cb), world);
+    btnStop = glade_xml_get_widget(gladexml, "btnLogStop");
+    g_signal_connect(G_OBJECT(btnStop), "clicked",
+                     G_CALLBACK(we_log_stop_cb), world);
+  } else {
+    /* These parts make no sense for the default world, so let's hide them */
+    GtkWidget *vbox_log_status;
+    GtkWidget *vbox_log_file;
+
+    vbox_log_status = glade_xml_get_widget(gladexml, "vbox_log_status");
+    vbox_log_file   = glade_xml_get_widget(gladexml, "vbox_log_file");
+
+    g_object_set(G_OBJECT(vbox_log_status), "no-show-all", TRUE, NULL);
+    g_object_set(G_OBJECT(vbox_log_file),   "no-show-all", TRUE, NULL);
+  }
 
   return panel;
 }
@@ -182,13 +194,28 @@ static
 void
 we_log_start_cb(GtkWidget *widget, gpointer data)
 {
-  World *world = (World *) data;
-  gchar *errmsg = NULL;
+  World           *world = (World *) data;
+  gint             lines = 0;
+  gchar           *errmsg = NULL;
+  GladeXML        *gladexml;
+  GtkToggleButton *chkLogIncludePrevious;
+  GtkSpinButton   *spnLogPreviousLines;
 
   /* Copy parameters in the World structure */
   update_we_logging_parameters(world);
 
-  if (start_log(world, &errmsg)) {
+  /* Has the user asked for inclusion of lines in the window buffer? */
+  gladexml              = glade_get_widget_tree(world->dlgEditWorld);
+  chkLogIncludePrevious
+    = GTK_TOGGLE_BUTTON(glade_xml_get_widget(gladexml, "chkLogIncludePrevious"));
+  spnLogPreviousLines
+    = GTK_SPIN_BUTTON(glade_xml_get_widget(gladexml, "spnLogPreviousLines"));
+
+  if (gtk_toggle_button_get_active(chkLogIncludePrevious)) {
+    lines = (int) gtk_spin_button_get_value(spnLogPreviousLines);
+  }
+
+  if (start_log(world, lines, &errmsg)) {
     fill_we_logging_tab(world, NULL);
   } else {
     GtkWidget *msgdlg = gtk_message_dialog_new(GTK_WINDOW(world->dlgEditWorld),
