@@ -1,7 +1,7 @@
-/* $Id: we_macros.c,v 1.15 2008/03/23 21:09:23 ekalin Exp $ */
+/* $Id: we_macros.c 1016 2009-02-02 12:10:40Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2008 Eduardo M Kalinowski <ekalin@gmail.com>
+ * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -419,9 +419,9 @@ edit_macro(World *world, Macro *macro)
   if (!macro->owner_plugin) {
     GtkWidget *label;
 
-    label = glade_xml_get_widget(gladexml, "lblNote");
+    label = glade_xml_get_widget(gladexml, "lblNoteMacro");
     gtk_widget_destroy(label);
-    label = glade_xml_get_widget(gladexml, "lblNoteText");
+    label = glade_xml_get_widget(gladexml, "lblNoteTextMacro");
     gtk_widget_destroy(label);
   }
 
