@@ -1,7 +1,7 @@
-/* $Id: worldgui.c,v 1.55 2008/03/23 21:09:15 ekalin Exp $ */
+/* $Id: worldgui.c 1025 2009-02-11 16:56:47Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2008 Eduardo M Kalinowski <ekalin@gmail.com>
+ * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -644,7 +644,7 @@ keypress_cb(GtkWidget   *widget,
       menu_perl_run_cb(NULL, "$world->prev");
       return TRUE;
     }
-    return FALSE;
+    break;
 
   case GDK_Right:
   case GDK_KP_Right:
@@ -652,7 +652,7 @@ keypress_cb(GtkWidget   *widget,
       menu_perl_run_cb(NULL, "$world->next");
       return TRUE;
     }
-    return FALSE;
+    break;
 
   case GDK_Tab:
   case GDK_ISO_Left_Tab:
@@ -664,7 +664,7 @@ keypress_cb(GtkWidget   *widget,
       }
       return TRUE;
     }
-    return FALSE;
+    break;
 
   case '1': case '2': case '3': case '4': case '5':
   case '6': case '7': case '8': case '9':
@@ -676,7 +676,7 @@ keypress_cb(GtkWidget   *widget,
       }
       return TRUE;
     }
-    return FALSE;
+    break;
 
   case GDK_Up:
   case GDK_KP_Up:
@@ -715,23 +715,11 @@ keypress_cb(GtkWidget   *widget,
                                adjustment->upper - adjustment->page_size);
       gtk_adjustment_changed(adjustment);
       return TRUE;
-    } else {
-      return FALSE;
     }
-
-  /* Let Shift+Return and CTRL+Return also work as return */
-  case GDK_Return:
-    if (evt->state & GDK_SHIFT_MASK || evt->state & GDK_CONTROL_MASK) {
-      activate_cb(world->gui->cmbEntry, gui);
-      return TRUE;
-    }
-    return FALSE;
-
-  default:
-    return check_macros(world, evt->keyval, evt->state);
+    break;
   }
 
-  return FALSE;
+  return check_macros(world, evt->keyval, evt->state);
 }
 
 
