@@ -1,7 +1,7 @@
-/* $Id: worlds.c,v 1.98 2008/03/23 21:09:13 ekalin Exp $ */
+/* $Id: worlds.c 1019 2009-02-03 19:31:11Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2008 Eduardo M Kalinowski <ekalin@gmail.com>
+ * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,6 +32,7 @@
 
 #include "kildclient.h"
 #include "perlscript.h"
+#include "net.h"
 
 
 /********************
@@ -44,6 +45,7 @@ typedef enum {
   KC_HOST,
   KC_PORT,
   KC_CHARSET,
+  KC_PROXY,
   KC_AUTO_LOGON,
   KC_CHARACTER,
   KC_PASSWORD,
@@ -61,12 +63,14 @@ typedef enum {
   KC_ANSICOLOR,
   KC_BACKGROUND,
   KC_SCROLL,
+  KC_NAMEDISPLAY,
   KC_LOG,
   KC_LOGFILE,
   KC_LOGTIMEFORMAT,
   KC_FLOOD_PREVENTION,
   KC_REPEAT_COMMANDS,
   KC_COMMAND_ECHO,
+  KC_NEVER_HIDE_INPUT,
   KC_STORE_COMMANDS,
   KC_COMMAND_HISTORY,
   KC_AUTO_COMPLETION,
@@ -152,7 +156,7 @@ static void copy_text(char **dest, const char *orig, gsize len);
 static void read_color(GdkColor *color,
                        const char *text, gsize text_len,
                        GError **error);
-static void write_color(FILE *fp, char *name, int idx, GdkColor *color);
+static void write_color(GString *str, char *name, int idx, GdkColor *color);
 
 
 /********************
@@ -214,6 +218,8 @@ create_new_world(gboolean load_defaults)
                                                               G_TYPE_STRING,
                                                               G_TYPE_STRING));
 
+  world->proxy.type = PROXY_USE_GLOBAL;
+
   if (!load_defaults) {
     return world;
   }
@@ -241,12 +247,15 @@ create_new_world(gboolean load_defaults)
   memcpy(world->defback,     default_world->defback,     sizeof(GdkColor));
   memcpy(world->defboldback, default_world->defboldback, sizeof(GdkColor));
   memcpy(world->ansicolors,  default_world->ansicolors,  16*sizeof(GdkColor));
+  world->name_display_style   = default_world->name_display_style;
+  world->name_display_pattern = g_strdup(default_world->name_display_pattern);
 
   world->scrollOutput        = default_world->scrollOutput;
   world->buffer_lines        = default_world->buffer_lines;
 
   world->repeat_commands     = default_world->repeat_commands;
   world->cmd_echo            = default_world->cmd_echo;
+  world->never_hide_input    = default_world->never_hide_input;
   world->store_commands      = default_world->store_commands;
   world->commands_to_save    = default_world->commands_to_save;
   world->input_n_lines       = default_world->input_n_lines;
@@ -314,12 +323,15 @@ create_default_world(void)
   memcpy(default_world->defback,     &defback,     sizeof(GdkColor));
   memcpy(default_world->defboldback, &defboldback, sizeof(GdkColor));
   memcpy(default_world->ansicolors,  &defansi,     16*sizeof(GdkColor));
+  default_world->name_display_style   = NAME_DISPLAY_WORLD;
+  default_world->name_display_pattern = g_strdup("%Kw [%Kc]");
 
   default_world->scrollOutput        = FALSE;
   default_world->buffer_lines        = 2000;
 
   default_world->repeat_commands     = FALSE;
   default_world->cmd_echo            = TRUE;
+  default_world->never_hide_input    = FALSE;
   default_world->store_commands      = TRUE;
   default_world->commands_to_save    = 50;
   default_world->input_n_lines       = 1;
@@ -374,7 +386,8 @@ free_world(World *world)
   }
 
   g_free(world->name);
-  g_free(world->new_text_str);
+  g_free(world->display_name);
+  g_free(world->new_text_name);
   g_free(world->file);
   g_free(world->charset);
   g_free(world->host);
@@ -383,6 +396,9 @@ free_world(World *world)
 
   g_free(world->pbuffer);
 
+  if (world->character_used) {
+    gtk_tree_row_reference_free(world->character_used);
+  }
   g_object_unref(world->logon_characters);
 
   g_free(world->terminalfont);
@@ -394,6 +410,10 @@ free_world(World *world)
   g_free(world->defback);
   g_free(world->defboldback);
   g_free(world->ansicolors);
+  g_free(world->name_display_pattern);
+
+  proxy_info_free(&world->proxy);
+  proxy_info_free(&world->proxy_used);
 
   g_free(world->scriptfile);
 
@@ -569,11 +589,6 @@ load_world_from_file(const char *file,
 
   fclose(fp);
 
-  if (!parser->forWorldSelector && world->name) {
-    world->new_text_str = g_strdup_printf("<span foreground=\"red\">%s</span>",
-                                          world->name);
-  }
-
   g_markup_parse_context_free(parseContext);
   g_free(parser);
 
@@ -630,6 +645,32 @@ xml_start_element(GMarkupParseContext *context,
       parser->currText = &parser->world->charset;
       parser->state = KC_CHARSET;
       g_free(parser->world->charset);
+    } else if (strcmp(element_name, "proxy") == 0) {
+      parser->state = KC_PROXY;
+      if (parser->forWorldSelector) {
+        break;
+      }
+
+      while (*attribute_names) {
+        if (strcmp(*attribute_names, "type") == 0) {
+          parser->world->proxy.type = atoi(*attribute_values);
+        } else if (strcmp(*attribute_names, "server") == 0) {
+          parser->world->proxy.server = g_strdup(*attribute_values);
+        } else if (strcmp(*attribute_names, "port") == 0) {
+          parser->world->proxy.port = g_strdup(*attribute_values);
+        } else if (strcmp(*attribute_names, "user") == 0) {
+          parser->world->proxy.user = g_strdup(*attribute_values);
+        } else if (strcmp(*attribute_names, "password") == 0) {
+          parser->world->proxy.password = g_strdup(*attribute_values);
+        } else {
+          g_set_error(error,
+                      G_MARKUP_ERROR, G_MARKUP_ERROR_UNKNOWN_ELEMENT,
+                      _("Invalid attribute '%s' for element '%s'"),
+                      *attribute_names, "proxy");
+        }
+        ++attribute_names;
+        ++attribute_values;
+      }
     } else if (strcmp(element_name, "display") == 0) {
       parser->state = KC_DISPLAY;
     } else if (strcmp(element_name, "scroll") == 0) {
@@ -652,6 +693,26 @@ xml_start_element(GMarkupParseContext *context,
         ++attribute_names;
         ++attribute_values;
       }
+    } else if (strcmp(element_name, "name-display") == 0) {
+      parser->state = KC_NAMEDISPLAY;
+      if (parser->forWorldSelector) {
+        break;
+      }
+
+      while (*attribute_names) {
+        if (strcmp(*attribute_names, "style") == 0) {
+          parser->world->name_display_style = atoi(*attribute_values);
+        } else if (strcmp(*attribute_names, "pattern") == 0) {
+          parser->world->name_display_pattern = g_strdup(*attribute_values);
+        } else {
+          g_set_error(error,
+                      G_MARKUP_ERROR, G_MARKUP_ERROR_UNKNOWN_ELEMENT,
+                      _("Invalid attribute '%s' for element '%s'"),
+                      *attribute_names, "name-display");
+        }
+        ++attribute_names;
+        ++attribute_values;
+      }
     } else if (strcmp(element_name, "log") == 0) {
       parser->state = KC_LOG;
       if (parser->forWorldSelector) {
@@ -749,6 +810,24 @@ xml_start_element(GMarkupParseContext *context,
         ++attribute_names;
         ++attribute_values;
       }
+    } else if (strcmp(element_name, "never-hide-input") == 0) {
+      parser->state = KC_NEVER_HIDE_INPUT;
+      if (parser->forWorldSelector) {
+        break;
+      }
+
+      while (*attribute_names) {
+        if (strcmp(*attribute_names, "enabled") == 0) {
+          parser->world->never_hide_input = atoi(*attribute_values);
+        } else {
+          g_set_error(error,
+                      G_MARKUP_ERROR, G_MARKUP_ERROR_UNKNOWN_ELEMENT,
+                      _("Invalid attribute '%s' for element '%s'"),
+                      *attribute_names, "never-hide-input");
+        }
+        ++attribute_names;
+        ++attribute_values;
+      }
     } else if (strcmp(element_name, "store-commands") == 0) {
       parser->state = KC_STORE_COMMANDS;
       if (parser->forWorldSelector) {
@@ -1419,12 +1498,14 @@ xml_end_element(GMarkupParseContext *context,
   case KC_HOST:
   case KC_PORT:
   case KC_CHARSET:
+  case KC_PROXY:
   case KC_AUTO_LOGON:
   case KC_DISPLAY:
   case KC_SCROLL:
   case KC_FLOOD_PREVENTION:
   case KC_REPEAT_COMMANDS:
   case KC_COMMAND_ECHO:
+  case KC_NEVER_HIDE_INPUT:
   case KC_STORE_COMMANDS:
   case KC_COMMAND_HISTORY:
   case KC_AUTO_COMPLETION:
@@ -1432,6 +1513,7 @@ xml_end_element(GMarkupParseContext *context,
   case KC_INPUT_LINES:
   case KC_INPUT_SPELL:
   case KC_CONFIRM_DELETE:
+  case KC_NAMEDISPLAY:
   case KC_LOG:
   case KC_PLUGIN_ITEMS:
   case KC_SCRIPTFILE:
@@ -1703,54 +1785,64 @@ copy_text(char **dest, const char *orig, gsize len)
 void
 save_world_to_file(World *world)
 {
-  FILE       *fp;
-  int         i;
-  GtkTreeIter iter;
+  int          i;
+  GtkTreeIter  iter;
+  GString     *file_contents;
+  GError      *error = NULL;
 
   if (!world->file) {
     fprintf(stderr, "Error: null filename\n");
     return;
   }
 
-  create_kildclient_directory();
-
-  fp = fopen(world->file, "w");
-  if (!fp) {
-    GtkWidget *dlg = gtk_message_dialog_new(GTK_WINDOW(wndMain),
-                                            GTK_DIALOG_DESTROY_WITH_PARENT,
-                                            GTK_MESSAGE_ERROR,
-                                            GTK_BUTTONS_CLOSE,
-                                            _("Could not open file '%s': %s"),
-                                            world->file,
-                                            g_strerror(errno));
-    gtk_window_set_title(GTK_WINDOW(dlg), _("KildClient Error"));
-    gtk_dialog_run(GTK_DIALOG(dlg));
-    gtk_widget_destroy(dlg);
-    return;
-  }
+  file_contents = g_string_sized_new(4096);
 
   /* Header */
-  fprintf(fp, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
-              "<!DOCTYPE kcworld SYSTEM \"kcworld.dtd\">\n\n"
-              "<kcworld>\n");
+  g_string_append(file_contents,
+                  "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
+                  "<!DOCTYPE kcworld SYSTEM \"kcworld.dtd\">\n\n"
+                  "<kcworld>\n");
 
   /* General data */
   if (world != default_world) {
-    fprintf_escaped(fp,
-                    "  <name>%s</name>\n"
-                    "  <host tls=\"%d\">%s</host>\n"
-                    "  <port>%s</port>\n",
-                    world->name,
-                    world->use_tls,
-                    world->host,
-                    world->port);
+    kc_g_string_append_escaped(file_contents,
+                               "  <name>%s</name>\n"
+                               "  <host tls=\"%d\">%s</host>\n"
+                               "  <port>%s</port>\n",
+                               world->name,
+                               world->use_tls,
+                               world->host,
+                               world->port);
   }
-  fprintf_escaped(fp, "  <charset>%s</charset>\n", world->charset);
+  kc_g_string_append_escaped(file_contents,
+                             "  <charset>%s</charset>\n", world->charset);
+
+  /* Proxy */
+  g_string_append_printf(file_contents,
+                         "  <proxy type=\"%d\" ", world->proxy.type);
+  if (world->proxy.server && strcmp(world->proxy.server, "") != 0) {
+    kc_g_string_append_escaped(file_contents,
+                               "server=\"%s\" ", world->proxy.server);
+  }
+  if (world->proxy.port && strcmp(world->proxy.port, "") != 0) {
+    kc_g_string_append_escaped(file_contents,
+                               "port=\"%s\" ", world->proxy.port);
+  }
+  if (world->proxy.user && strcmp(world->proxy.user, "") != 0) {
+    kc_g_string_append_escaped(file_contents,
+                               "user=\"%s\" ", world->proxy.user);
+  }
+  if (world->proxy.password && strcmp(world->proxy.password, "") != 0) {
+    kc_g_string_append_escaped(file_contents,
+                               "password=\"%s\" ", world->proxy.password);
+  }
+  g_string_append(file_contents, "/>\n");
 
   /* Auto-logon */
   if (world != default_world) {
-    fprintf(fp, "  <auto-logon style=\"%d\">\n",
-            world->connection_style);
+    g_string_append_printf(file_contents,
+                           "  <auto-logon style=\"%d\">\n",
+                           world->connection_style);
     if (gtk_tree_model_get_iter_first(world->logon_characters, &iter)) {
       do {
         gchar *character;
@@ -1760,102 +1852,126 @@ save_world_to_file(World *world)
                            LOGON_CHAR, &character,
                            LOGON_PASS,  &password,
                            -1);
-        fprintf_escaped(fp, "    <character>%s</character>\n", character);
-        fprintf_escaped(fp, "    <password>%s</password>\n",   password);
+        kc_g_string_append_escaped(file_contents,
+                                   "    <character>%s</character>\n",
+                                   character);
+        kc_g_string_append_escaped(file_contents,
+                                   "    <password>%s</password>\n",
+                                   password);
         g_free(character);
         g_free(password);
       } while (gtk_tree_model_iter_next(world->logon_characters, &iter));
     }
-    fprintf(fp, "  </auto-logon>\n");
+    g_string_append(file_contents, "  </auto-logon>\n");
   }
 
   /* Fonts, Colors, Status Bar and background */
-  fprintf(fp, "  <display>\n");
-  fprintf_escaped(fp,
-                  "    <terminalfont>%s</terminalfont>\n"
-                  "    <entryfont>%s</entryfont>\n"
-                  "    <usebold>%d</usebold>\n"
-                  "    <wrap enabled=\"%d\"/>\n"
-                  "    <statusfont>%s</statusfont>\n"
-                  "    <statusbar ctime=\"%d\" itime=\"%d\" itime-behavior=\"%d\"/>\n",
-                  world->terminalfont,
-                  world->entryfont,
-                  world->usebold,
-                  world->wrap,
-                  world->statusfont,
-                  world->ctime_type,
-                  world->itime_type,
-                  world->itime_reset_activate);
+  g_string_append(file_contents, "  <display>\n");
+  kc_g_string_append_escaped(file_contents,
+                             "    <terminalfont>%s</terminalfont>\n"
+                             "    <entryfont>%s</entryfont>\n"
+                             "    <usebold>%d</usebold>\n"
+                             "    <wrap enabled=\"%d\"/>\n"
+                             "    <statusfont>%s</statusfont>\n"
+                             "    <statusbar ctime=\"%d\" itime=\"%d\" itime-behavior=\"%d\"/>\n",
+                             world->terminalfont,
+                             world->entryfont,
+                             world->usebold,
+                             world->wrap,
+                             world->statusfont,
+                             world->ctime_type,
+                             world->itime_type,
+                             world->itime_reset_activate);
 
-  write_color(fp, "defforecolor",     0, world->deffore);
-  write_color(fp, "defbackcolor",     0, world->defback);
-  write_color(fp, "defboldcolor",     0, world->defbold);
-  write_color(fp, "defboldbackcolor", 0, world->defboldback);
+  write_color(file_contents, "defforecolor",     0, world->deffore);
+  write_color(file_contents, "defbackcolor",     0, world->defback);
+  write_color(file_contents, "defboldcolor",     0, world->defbold);
+  write_color(file_contents, "defboldbackcolor", 0, world->defboldback);
   for (i = 0; i < 16; ++i) {
-    write_color(fp, "ansicolor", i, world->ansicolors);
+    write_color(file_contents, "ansicolor", i, world->ansicolors);
   }
 
-  fprintf(fp, "  </display>\n");
+  g_string_append(file_contents, "  </display>\n");
 
   /* Command entry */
-  fprintf_escaped(fp,
-                  "  <repeat-commands enabled=\"%d\" />\n"
-                  "  <command-echo enabled=\"%d\" />\n"
-                  "  <store-commands enabled=\"%d\" />\n"
-                  "  <command-history size=\"%d\" />\n"
-                  "  <auto-completion enabled=\"%d\" minprefix=\"%d\" />\n"
-                  "  <command-separator separator=\"%s\" />\n"
-                  "  <input-lines size=\"%d\" save=\"%d\" />\n",
-                  world->repeat_commands,
-                  world->cmd_echo,
-                  world->store_commands,
-                  world->commands_to_save,
-                  world->autocompletion,
-                  world->autocompl_minprefix,
-                  world->command_separator,
-                  world->input_n_lines,
-                  world->input_n_lines_saved);
-  fprintf(fp, "  <input-spell active=\"%d\"", world->spell);
+  kc_g_string_append_escaped(file_contents,
+                             "  <repeat-commands enabled=\"%d\" />\n"
+                             "  <command-echo enabled=\"%d\" />\n"
+                             "  <never-hide-input enabled=\"%d\" />\n"
+                             "  <store-commands enabled=\"%d\" />\n"
+                             "  <command-history size=\"%d\" />\n"
+                             "  <auto-completion enabled=\"%d\" minprefix=\"%d\" />\n"
+                             "  <command-separator separator=\"%s\" />\n"
+                             "  <input-lines size=\"%d\" save=\"%d\" />\n",
+                             world->repeat_commands,
+                             world->cmd_echo,
+                             world->never_hide_input,
+                             world->store_commands,
+                             world->commands_to_save,
+                             world->autocompletion,
+                             world->autocompl_minprefix,
+                             world->command_separator,
+                             world->input_n_lines,
+                             world->input_n_lines_saved);
+  g_string_append_printf(file_contents,
+                         "  <input-spell active=\"%d\"", world->spell);
   if (world->spell_language) {
-    fprintf_escaped(fp, " language=\"%s\"", world->spell_language);
+    kc_g_string_append_escaped(file_contents,
+                               " language=\"%s\"", world->spell_language);
   }
-  fprintf(fp, " />\n");
+  g_string_append(file_contents, " />\n");
 
   /* Anti flood */
-  fprintf(fp, "  <flood-prevention enabled=\"%d\" max-repeat=\"%d\">",
-          world->flood_prevention,
-          world->max_equal_commands);
-  fprintf_escaped(fp, "%s</flood-prevention>\n",
-                  world->flood_prevention_command);
+  g_string_append_printf(file_contents,
+                         "  <flood-prevention enabled=\"%d\" max-repeat=\"%d\">",
+                         world->flood_prevention,
+                         world->max_equal_commands);
+  kc_g_string_append_escaped(file_contents,
+                             "%s</flood-prevention>\n",
+                             world->flood_prevention_command);
 
   /* Scrolling */
-  fprintf(fp, "  <scroll on-output=\"%d\" lines=\"%ld\" />\n",
-          world->scrollOutput, world->buffer_lines);
+  g_string_append_printf(file_contents,
+                         "  <scroll on-output=\"%d\" lines=\"%ld\" />\n",
+                         world->scrollOutput, world->buffer_lines);
+
+  /* Name display */
+  g_string_append_printf(file_contents,
+                         "  <name-display style=\"%d\" pattern=\"%s\" />\n",
+                         world->name_display_style,
+                         world->name_display_pattern);
 
   /* Logging */
-  fprintf(fp, "  <log autostart=\"%d\" addtime=\"%d\">\n",
-          world->log_autostart, world->log_add_time);
+  g_string_append_printf(file_contents,
+                         "  <log autostart=\"%d\" addtime=\"%d\">\n",
+                         world->log_autostart, world->log_add_time);
   if (world->log_file_name) {
-    fprintf_escaped(fp, "    <logfile>%s</logfile>\n", world->log_file_name);
+    kc_g_string_append_escaped(file_contents,
+                               "    <logfile>%s</logfile>\n",
+                               world->log_file_name);
   }
   if (world->log_timeformat) {
-    fprintf_escaped(fp, "    <logtimeformat>%s</logtimeformat>\n",
-                    world->log_timeformat);
+    kc_g_string_append_escaped(file_contents,
+                               "    <logtimeformat>%s</logtimeformat>\n",
+                               world->log_timeformat);
   }
-  fprintf(fp, "  </log>\n");
+  g_string_append(file_contents, "  </log>\n");
 
   /* Delete confirmation in GUI Editors */
-  fprintf(fp, "  <confirm-delete confirm=\"%d\" />\n",
-          world->confirm_delete);
+  g_string_append_printf(file_contents,
+                         "  <confirm-delete confirm=\"%d\" />\n",
+                         world->confirm_delete);
 
   /* Plugin items in GUI Editors */
-  fprintf(fp, "  <plugin-items show=\"%d\" />\n",
-          world->show_plugin_items);
+  g_string_append_printf(file_contents,
+                         "  <plugin-items show=\"%d\" />\n",
+                         world->show_plugin_items);
 
   /* Script file */
   if (world->scriptfile && strcmp(world->scriptfile, "") != 0) {
-    fprintf_escaped(fp, "  <scriptfile>%s</scriptfile>\n",
-                    world->scriptfile);
+    kc_g_string_append_escaped(file_contents,
+                               "  <scriptfile>%s</scriptfile>\n",
+                               world->scriptfile);
   }
 
   /* Startup plugins */
@@ -1864,15 +1980,16 @@ save_world_to_file(World *world)
     GtkTreeIter  iter;
     gchar       *file;
 
-    fprintf(fp, "  <plugins>\n");
+    g_string_append(file_contents, "  <plugins>\n");
     gtk_tree_model_get_iter_first(world->startup_plugins, &iter);
     do {
       gtk_tree_model_get(world->startup_plugins, &iter,
                          SPLUGIN_FILE, &file, -1);
-      fprintf_escaped(fp, "    <plugin>%s</plugin>\n", file);
+      kc_g_string_append_escaped(file_contents,
+                                 "    <plugin>%s</plugin>\n", file);
       g_free(file);
     } while (gtk_tree_model_iter_next(world->startup_plugins, &iter));
-    fprintf(fp, "  </plugins>\n");
+    g_string_append(file_contents, "  </plugins>\n");
   }
 
   /* Triggers */
@@ -1880,7 +1997,7 @@ save_world_to_file(World *world)
     GSList  *trigptr = world->triggers;
     Trigger *trigger;
 
-    fprintf(fp, "  <triggers>\n");
+    g_string_append(file_contents, "  <triggers>\n");
 
     while (trigptr) {
       trigger = (Trigger *) trigptr->data;
@@ -1890,10 +2007,10 @@ save_world_to_file(World *world)
         continue;
       }
 
-      save_trigger(fp, trigger);
+      save_trigger(file_contents, trigger);
     }
 
-    fprintf(fp, "  </triggers>\n");
+    g_string_append(file_contents, "  </triggers>\n");
   }
 
   /* Aliases */
@@ -1901,7 +2018,7 @@ save_world_to_file(World *world)
     GSList *aliasptr = world->aliases;
     Alias  *alias;
 
-    fprintf(fp, "  <aliases>\n");
+    g_string_append(file_contents, "  <aliases>\n");
 
     while (aliasptr) {
       alias = (Alias *) aliasptr->data;
@@ -1911,10 +2028,10 @@ save_world_to_file(World *world)
         continue;
       }
 
-      save_alias(fp, alias);
+      save_alias(file_contents, alias);
     }
 
-    fprintf(fp, "  </aliases>\n");
+    g_string_append(file_contents, "  </aliases>\n");
   }
 
   /* Macros */
@@ -1922,7 +2039,7 @@ save_world_to_file(World *world)
     GSList *macroptr = world->macros;
     Macro  *macro;
 
-    fprintf(fp, "  <macros>\n");
+    g_string_append(file_contents, "  <macros>\n");
 
     while (macroptr) {
       macro = (Macro *) macroptr->data;
@@ -1932,9 +2049,9 @@ save_world_to_file(World *world)
         continue;
       }
 
-      save_macro(fp, macro);
+      save_macro(file_contents, macro);
     }
-    fprintf(fp, "  </macros>\n");
+    g_string_append(file_contents, "  </macros>\n");
   }
 
   /* Timers */
@@ -1942,7 +2059,7 @@ save_world_to_file(World *world)
     GSList *timerptr = world->timers;
     Timer  *timer;
 
-    fprintf(fp, "  <timers>\n");
+    g_string_append(file_contents, "  <timers>\n");
 
     while (timerptr) {
       timer = (Timer *) timerptr->data;
@@ -1952,37 +2069,54 @@ save_world_to_file(World *world)
         continue;
       }
 
-      save_timer(fp, timer);
+      save_timer(file_contents, timer);
     }
 
-    fprintf(fp, "  </timers>\n");
+    g_string_append(file_contents, "  </timers>\n");
   }
 
   /* Permanent variables */
   if (world->permanent_variables) {
     GSList *varptr = world->permanent_variables;
 
-    fprintf(fp, "  <variables>\n");
+    g_string_append(file_contents, "  <variables>\n");
 
     while (varptr) {
-      fprintf(fp, "    <variable>%s</variable>\n",
+      g_string_append_printf(file_contents, "    <variable>%s</variable>\n",
               (char *) varptr->data);
       varptr = varptr->next;
     }
 
-    fprintf(fp, "  </variables>\n");
+    g_string_append(file_contents, "  </variables>\n");
   }
 
   /* Hooks */
-  save_hooks(fp, world);
+  save_hooks(file_contents, world);
 
   /* Protocols */
-  fprintf(fp, "  <protocols>\n");
-  fprintf(fp, "    <mccp behavior=\"%d\"/>\n", world->mccp_behavior);
-  fprintf(fp, "  </protocols>\n");
+  g_string_append(file_contents, "  <protocols>\n");
+  g_string_append_printf(file_contents,
+                         "    <mccp behavior=\"%d\"/>\n", world->mccp_behavior);
+  g_string_append(file_contents, "  </protocols>\n");
 
-  fprintf(fp, "</kcworld>\n");
-  fclose(fp);
+  g_string_append(file_contents, "</kcworld>\n");
+
+  create_kildclient_directory();
+  if (!g_file_set_contents(world->file,
+                           file_contents->str, file_contents->len,
+                           &error)) {
+    GtkWidget *dlg = gtk_message_dialog_new(GTK_WINDOW(wndMain),
+                                            GTK_DIALOG_DESTROY_WITH_PARENT,
+                                            GTK_MESSAGE_ERROR,
+                                            GTK_BUTTONS_CLOSE,
+                                            _("Could not open file '%s': %s"),
+                                            world->file,
+                                            error->message);
+    gtk_window_set_title(GTK_WINDOW(dlg), _("KildClient Error"));
+    gtk_dialog_run(GTK_DIALOG(dlg));
+    gtk_widget_destroy(dlg);
+  }
+  g_string_free(file_contents, TRUE);
 }
 
 
@@ -2033,11 +2167,11 @@ load_command_history(World *world)
 void
 save_command_history(World *world)
 {
-  FILE        *fp;
   gchar       *path;
   GtkTreeIter  iter;
   gchar       *command;
   int          start_pos;
+  GString     *file_contents;
 
   if (!world->store_commands)
     return;
@@ -2047,13 +2181,7 @@ save_command_history(World *world)
     return;
   }
 
-  path = g_strdup_printf("%s.hst", world->file);
-
-  fp = fopen(path, "w");
-  g_free(path);
-  if (!fp) {
-    return;
-  }
+  file_contents = g_string_sized_new(1024);
 
   /* Define start position. Most of the times saving starts at the
      second (pos 1) command because the first represents the command
@@ -2069,24 +2197,28 @@ save_command_history(World *world)
                                     NULL, start_pos)) {
     do {
       command = cmdhistory_get_command(world->cmd_list, &iter);
-      fprintf(fp, "%s\n", command);
+      g_string_append(file_contents, command);
+      g_string_append_c(file_contents, '\n');
       g_free(command);
     } while (gtk_tree_model_iter_next(world->cmd_list, &iter));
   }
 
-  fclose(fp);
+  path = g_strdup_printf("%s.hst", world->file);
+  g_file_set_contents(path, file_contents->str, file_contents->len, NULL);
+  g_free(path);
+  g_string_free(file_contents, TRUE);
 }
 
 
 void
-fprintf_escaped(FILE *fp, const char *template, ...)
+kc_g_string_append_escaped(GString *gstr, const char *template, ...)
 {
   va_list  ap;
   char    *str;
 
   va_start(ap, template);
   str = g_markup_vprintf_escaped(template, ap);
-  fputs(str, fp);
+  g_string_append(gstr, str);
   g_free(str);
 }
 
@@ -2118,11 +2250,11 @@ read_color(GdkColor *color, const char *text, gsize text_len, GError **error)
 
 static
 void
-write_color(FILE *fp, char *name, int idx, GdkColor *color)
+write_color(GString *str, char *name, int idx, GdkColor *color)
 {
-  fprintf(fp, "    <%s>%d %d %d %d</%s>\n", name, idx,
-          color[idx].red, color[idx].green, color[idx].blue,
-          name);
+  g_string_append_printf(str, "    <%s>%d %d %d %d</%s>\n", name, idx,
+                         color[idx].red, color[idx].green, color[idx].blue,
+                         name);
 }
 
 
@@ -2132,7 +2264,7 @@ mark_as_offline(World *world)
   gchar *str;
 
   str = g_strdup_printf("<span foreground=\"grey\">%s</span>",
-                        world->name);
+                        world->display_name);
   gtk_label_set_markup(world->gui->lblNotebook, str);
   g_free(str);
 
@@ -2142,3 +2274,95 @@ mark_as_offline(World *world)
 
   currentWorld = world;
 }
+
+
+void
+prepare_display_name(World *world)
+{
+  gchar *character;
+
+  g_free(world->display_name);
+  g_free(world->new_text_name);
+
+  if (world->character_used == NULL || world->connection_style == NONE) {
+    character = g_strdup("");
+  } else {
+    GtkTreePath *path;
+    GtkTreeIter  iter;
+
+    path = gtk_tree_row_reference_get_path(world->character_used);
+    if (!path) {
+      gtk_tree_row_reference_free(world->character_used);
+      world->character_used = NULL;
+      character = g_strdup("");
+    }
+
+    gtk_tree_model_get_iter(world->logon_characters,
+                            &iter, path);
+    gtk_tree_model_get(world->logon_characters, &iter,
+                       LOGON_CHAR, &character,
+                       -1);
+  }
+
+  switch (world->name_display_style) {
+  case NAME_DISPLAY_WORLD:
+    world->display_name = g_strdup(world->name);
+    break;
+
+  case NAME_DISPLAY_WORLD_CHAR:
+    world->display_name = g_strdup_printf("%s - %s", world->name, character);
+    break;
+
+  case NAME_DISPLAY_CHAR_WORLD:
+    world->display_name = g_strdup_printf("%s - %s", character, world->name);
+    break;
+
+  case NAME_DISPLAY_CUSTOM:
+    world->display_name = replace_kc_escapes(world->name_display_pattern,
+                                             world->name, character);
+    break;
+  }
+  world->new_text_name = g_strdup_printf("<span foreground=\"red\">%s</span>",
+                                         world->display_name);
+
+
+  g_free(character);
+}
+
+
+gchar *
+replace_kc_escapes(const gchar *pattern,
+                   const gchar *name,
+                   const gchar *character)
+{
+  static GRegex *reKw = NULL;
+  static GRegex *reKc = NULL;
+  gchar         *tmp1;
+  gchar         *tmp2;
+
+  /* If this is the first run, create the GRegex structure */
+  if (!reKw) {
+    reKw = g_regex_new("%Kw", 0, 0, NULL);
+  }
+  if (!reKc) {
+    reKc = g_regex_new("%Kc", 0, 0, NULL);
+  }
+
+  /* Substitute KC specific strings:
+     %Kw - World name
+     %Kc - Character name */
+  tmp1 = g_regex_replace_literal(reKw,
+                                 pattern, -1,
+                                 0,
+                                 name,
+                                 0, NULL);
+  tmp2 = g_regex_replace_literal(reKc,
+                                 tmp1, -1,
+                                 0,
+                                 character,
+                                 0, NULL);
+
+  g_free(tmp1);
+  return tmp2;
+}
+
