@@ -1,7 +1,7 @@
-/* $Id: worldselector.c,v 1.58 2008/05/03 13:34:08 ekalin Exp $ */
+/* $Id: worldselector.c 1019 2009-02-03 19:31:11Z ekalin $ */
 
 /*
- * Copyright (C) 2004-2008 Eduardo M Kalinowski <ekalin@gmail.com>
+ * Copyright (C) 2004-2009 Eduardo M Kalinowski <eduardo@kalinowski.com.br>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -116,13 +116,16 @@ get_world_to_connect()
 
       if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radManually))) {
         wrld->name = g_strdup(_("Unnamed world"));
-        wrld->new_text_str =
+        /* Since there is never a character in this case, it makes no
+           sense to use the default world settings for display name. */
+        wrld->display_name = g_strdup(wrld->name);
+        wrld->new_text_name =
           g_strdup_printf("<span foreground=\"red\">%s</span>",
-                          wrld->name);
+                          wrld->display_name);
         wrld->host = g_strdup(gtk_entry_get_text(GTK_ENTRY(txtHost)));
         portstr = gtk_entry_get_text(GTK_ENTRY(txtPort));
         wrld->port = g_strdup(portstr);
-        wrld->character_used = -1;    /* No auto-logon */
+        wrld->character_used = NULL;    /* No auto-logon */
         success = TRUE;
       } else {
         GtkTreeSelection *selection;
@@ -154,7 +157,18 @@ get_world_to_connect()
             free_world(wrld);
             wrld = NULL;
           } else {
-            wrld->character_used = charidx;
+            if (charidx == -1) {
+              wrld->character_used = NULL;
+            } else {
+              GtkTreePath         *path;
+              GtkTreeRowReference *ref;
+
+              path = gtk_tree_path_new_from_indices(charidx, -1);
+              ref  = gtk_tree_row_reference_new(wrld->logon_characters, path);
+
+              wrld->character_used = ref;
+              gtk_tree_path_free(path);
+            }
           }
         }
       }
